@@ -3,11 +3,11 @@
 //Plot styles
 enum
 {
-  PLOT_TYPE_LINES=0,
-  PLOT_TYPE_BARS,
-  PLOT_TYPE_STEPS,
-  PLOT_TYPE_STEM,
-  PLOT_TYPE_ENDOFENUM,
+  PLOT_TRACE_LINES=0,
+  PLOT_TRACE_BARS,
+  PLOT_TRACE_STEPS,
+  PLOT_TRACE_STEM,
+  PLOT_TRACE_ENDOFENUM
 };
 
 //Plot error types
@@ -33,14 +33,16 @@ enum
 	PROPERTY_TYPE_COLOUR,
 	PROPERTY_TYPE_STRING,
 	PROPERTY_TYPE_POINT3D,
-	PROPERTY_TYPE_CHOICE,
+	PROPERTY_TYPE_CHOICE
 };
 
-//!Plotting region ID codes
-enum{
-	REGION_LEFT_EXTEND=1,
-	REGION_MOVE,
-	REGION_RIGHT_EXTEND
+
+//!Movement types for plot
+enum
+{
+	REGION_MOVE_EXTEND_XMINUS,
+	REGION_MOVE_TRANSLATE_X,
+	REGION_MOVE_EXTEND_XPLUS
 };
 
 //!Data holder for colour as float
