@@ -2,6 +2,8 @@
 
 #include "../xmlHelper.h"
 
+#include "../translation.h"
+
 #include <algorithm>
 
 enum
@@ -15,6 +17,7 @@ enum
 	KEY_NOISETYPE,
 	KEY_ROTATE_ANGLE,
 	KEY_ROTATE_AXIS,
+	KEY_ORIGIN_VALUE
 };
 
 //Possible transform modes (scaling, rotation etc)
@@ -25,6 +28,7 @@ enum
 	MODE_ROTATE,
 	MODE_VALUE_SHUFFLE,
 	MODE_SPATIAL_NOISE,
+	MODE_TRANSLATE_VALUE,
 	MODE_ENUM_END
 };
 
@@ -104,6 +108,7 @@ DrawStreamData* TransformFilter::makeMarkerSphere(SelectionDevice<Filter>* &s) c
 {
 	//construct a new primitive, do not cache
 	DrawStreamData *drawData=new DrawStreamData;
+	drawData->parent=this;
 	//Add drawable components
 	DrawSphere *dS = new DrawSphere;
 	dS->setOrigin(vectorParams[0]);
@@ -287,6 +292,7 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 						{
 							//Set up scaling output ion stream 
 							IonStreamData *d=new IonStreamData;
+							d->parent=this;
 							const IonStreamData *src = (const IonStreamData *)dataIn[ui];
 
 							try
@@ -406,7 +412,8 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 						{
 							//Set up scaling output ion stream 
 							IonStreamData *d=new IonStreamData;
-
+							d->parent=this;
+							
 							const IonStreamData *src = (const IonStreamData *)dataIn[ui];
 							try
 							{
@@ -423,36 +430,36 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 							d->a = src->a;
 							d->ionSize = src->ionSize;
 							d->valueType=src->valueType;
-
+							
 							ASSERT(src->data.size() <= totalSize);
 							unsigned int curProg=NUM_CALLBACK;
 #ifdef _OPENMP
 							//Parallel version
 							bool spin=false;
-							#pragma omp parallel for shared(spin)
+#pragma omp parallel for shared(spin)
 							for(unsigned int ui=0;ui<src->data.size();ui++)
 							{
 								unsigned int thisT=omp_get_thread_num();
 								if(spin)
 									continue;
-
+								
 								if(!curProg--)
 								{
-									#pragma omp critical
+#pragma omp critical
 									{
-									n+=NUM_CALLBACK;
-									progress.filterProgress= (unsigned int)((float)(n)/((float)totalSize)*100.0f);
+										n+=NUM_CALLBACK;
+										progress.filterProgress= (unsigned int)((float)(n)/((float)totalSize)*100.0f);
 									}
-
-
+									
+									
 									if(thisT == 0)
 									{
 										if(!(*callback)())
 											spin=true;
 									}
 								}
-
-
+								
+								
 								//set the position for the given ion
 								d->data[ui].setPos((src->data[ui].getPosRef() - origin));
 								d->data[ui].setMassToCharge(src->data[ui].getMassToCharge());
@@ -462,13 +469,13 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 								delete d;
 								return ERR_CALLBACK_FAIL;
 							}
-
+							
 #else
 							//Single threaded version
 							size_t pos=0;
 							//Copy across the ions into the target
 							for(vector<IonHit>::const_iterator it=src->data.begin();
-								       it!=src->data.end(); ++it)
+								it!=src->data.end(); ++it)
 							{
 								//set the position for the given ion
 								d->data[pos].setPos((it->getPosRef() - origin));
@@ -509,6 +516,124 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 					}
 					break;
 				}
+				case MODE_TRANSLATE_VALUE:
+				{
+					//We are going to scale the incoming point data
+					//around the specified origin.
+					ASSERT(vectorParams.size() == 0);
+					ASSERT(scalarParams.size() == 1);
+					float origin =scalarParams[0];
+					size_t n=0;
+					switch(dataIn[ui]->getStreamType())
+					{
+						case STREAM_TYPE_IONS:
+						{
+							//Set up scaling output ion stream 
+							IonStreamData *d=new IonStreamData;
+							d->parent=this;
+							
+							const IonStreamData *src = (const IonStreamData *)dataIn[ui];
+							try
+							{
+								d->data.resize(src->data.size());
+							}
+							catch(std::bad_alloc)
+							{
+								delete d;
+								return ERR_NOMEM;
+							}
+							d->r = src->r;
+							d->g = src->g;
+							d->b = src->b;
+							d->a = src->a;
+							d->ionSize = src->ionSize;
+							d->valueType=src->valueType;
+							
+							ASSERT(src->data.size() <= totalSize);
+							unsigned int curProg=NUM_CALLBACK;
+#ifdef _OPENMP
+							//Parallel version
+							bool spin=false;
+#pragma omp parallel for shared(spin)
+							for(unsigned int ui=0;ui<src->data.size();ui++)
+							{
+								unsigned int thisT=omp_get_thread_num();
+								if(spin)
+									continue;
+								
+								if(!curProg--)
+								{
+#pragma omp critical
+									{
+										n+=NUM_CALLBACK;
+										progress.filterProgress= (unsigned int)((float)(n)/((float)totalSize)*100.0f);
+									}
+									
+									
+									if(thisT == 0)
+									{
+										if(!(*callback)())
+											spin=true;
+									}
+								}
+								
+								
+								//set the position for the given ion
+								d->data[ui].setPos((src->data[ui].getPosRef()));
+								d->data[ui].setMassToCharge(src->data[ui].getMassToCharge()+origin);
+							}
+							if(spin)
+							{			
+								delete d;
+								return ERR_CALLBACK_FAIL;
+							}
+							
+#else
+							//Single threaded version
+							size_t pos=0;
+							//Copy across the ions into the target
+							for(vector<IonHit>::const_iterator it=src->data.begin();
+								it!=src->data.end(); ++it)
+							{
+								//set the position for the given ion
+								d->data[pos].setPos((it->getPosRef()));
+								d->data[pos].setMassToCharge(it->getMassToCharge() + origin);
+								//update progress every CALLBACK ions
+								if(!curProg--)
+								{
+									n+=NUM_CALLBACK;
+									progress.filterProgress= (unsigned int)((float)(n)/((float)totalSize)*100.0f);
+									if(!(*callback)())
+									{
+										delete d;
+										return ERR_CALLBACK_FAIL;
+									}
+									curProg=NUM_CALLBACK;
+								}
+								pos++;
+							}
+							ASSERT(pos == d->data.size());
+#endif
+							ASSERT(d->data.size() == src->data.size());
+							if(cache)
+							{
+								d->cached=1;
+								filterOutputs.push_back(d);
+								cacheOK=true;
+							}
+							else
+								d->cached=0;
+							
+							getOut.push_back(d);
+							break;
+						}
+						default:
+							//Just copy across the ptr, if we are unfamiliar with this type
+							getOut.push_back(dataIn[ui]);	
+							break;
+					}
+					break;
+				}
 				case MODE_ROTATE:
 				{
 					Point3D origin=vectorParams[0];
@@ -520,6 +645,7 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 							const IonStreamData *src = (const IonStreamData *)dataIn[ui];
 							//Set up output ion stream 
 							IonStreamData *d=new IonStreamData;
+							d->parent=this;
 							try
 							{
 								d->data.resize(src->data.size());
@@ -619,6 +745,7 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 						{
 							//Set up scaling output ion stream 
 							IonStreamData *d=new IonStreamData;
+							d->parent=this;
 
 							const IonStreamData *src = (const IonStreamData *)dataIn[ui];
 							try
@@ -743,7 +870,7 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 	{
 		progress.step=1;
 		progress.filterProgress=0;
-		progress.stepName="Collate";
+		progress.stepName=TRANS("Collate");
 		progress.maxStep=3;
 		//we have to cross the streams (I thought that was bad?) 
 		//  - Each dataset is no longer independant, and needs to
@@ -751,6 +878,7 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 		
 		//Set up output ion stream 
 		IonStreamData *d=new IonStreamData;
+		d->parent=this;
 		
 		//TODO: Better output colouring/size
 		//Set up ion metadata
@@ -759,7 +887,7 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 		d->b = 0.5;
 		d->a = 0.5;
 		d->ionSize = 2.0;
-		d->valueType="Mass-to-Charge";
+		d->valueType=TRANS("Mass-to-Charge");
 
 		size_t n=0;
 		size_t curPos=0;
@@ -822,7 +950,7 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 	
 		progress.step=1;
 		progress.filterProgress=0;
-		progress.stepName="Shuffle";
+		progress.stepName=TRANS("Shuffle");
 		if(!(*callback)())
 		{
 			delete d;
@@ -838,7 +966,7 @@ unsigned int TransformFilter::refresh(const std::vector<const FilterStreamData *
 
 		progress.step=2;
 		progress.filterProgress=0;
-		progress.stepName="Shuffle";
+		progress.stepName=TRANS("Splice");
 		if(!(*callback)())
 		{
 			delete d;
@@ -896,23 +1024,24 @@ void TransformFilter::getProperties(FilterProperties &propertyList) const
 
 	string tmpStr;
 	vector<pair<unsigned int,string> > choices;
-	choices.push_back(make_pair((unsigned int) MODE_TRANSLATE,"Translate"));
-	choices.push_back(make_pair((unsigned int)MODE_SCALE,"Scale"));
-	choices.push_back(make_pair((unsigned int)MODE_ROTATE,"Rotate"));
-	choices.push_back(make_pair((unsigned int)MODE_VALUE_SHUFFLE,"Value Shuffle"));
-	choices.push_back(make_pair((unsigned int)MODE_SPATIAL_NOISE,"Spatial Noise"));
+	choices.push_back(make_pair((unsigned int) MODE_TRANSLATE,TRANS("Translate")));
+	choices.push_back(make_pair((unsigned int)MODE_SCALE,TRANS("Scale")));
+	choices.push_back(make_pair((unsigned int)MODE_ROTATE,TRANS("Rotate")));
+	choices.push_back(make_pair((unsigned int)MODE_VALUE_SHUFFLE,TRANS("Value Shuffle")));
+	choices.push_back(make_pair((unsigned int)MODE_SPATIAL_NOISE,TRANS("Spatial Noise")));
+	choices.push_back(make_pair((unsigned int) MODE_TRANSLATE_VALUE,TRANS("Translate Value")));
 	
 	tmpStr= choiceString(choices,transformMode);
 	choices.clear();
 	
-	s.push_back(make_pair(string("Mode"),tmpStr));
+	s.push_back(make_pair(string(TRANS("Mode")),tmpStr));
 	type.push_back(PROPERTY_TYPE_CHOICE);
 	keys.push_back(KEY_MODE);
 	
 	propertyList.data.push_back(s);
 	propertyList.types.push_back(type);
 	propertyList.keys.push_back(keys);
-	propertyList.keyNames.push_back("Type");
+	propertyList.keyNames.push_back(TRANS("Type"));
 	s.clear();type.clear();keys.clear();
 	
 	//non-translation transforms require a user to select an origin	
@@ -924,12 +1053,12 @@ void TransformFilter::getProperties(FilterProperties &propertyList) const
 		
 		tmpStr= choiceString(choices,originMode);
 
-		s.push_back(make_pair(string("Origin mode"),tmpStr));
+		s.push_back(make_pair(string(TRANS("Origin mode")),tmpStr));
 		type.push_back(PROPERTY_TYPE_CHOICE);
 		keys.push_back(KEY_ORIGINMODE);
 	
 		stream_cast(tmpStr,showOrigin);	
-		s.push_back(make_pair(string("Show marker"),tmpStr));
+		s.push_back(make_pair(string(TRANS("Show marker")),tmpStr));
 		type.push_back(PROPERTY_TYPE_BOOL);
 		keys.push_back(KEY_TRANSFORM_SHOWORIGIN);
 	}
@@ -945,27 +1074,42 @@ void TransformFilter::getProperties(FilterProperties &propertyList) const
 			
 			stream_cast(tmpStr,vectorParams[0]);
 			keys.push_back(KEY_ORIGIN);
-			s.push_back(make_pair("Translation", tmpStr));
+			s.push_back(make_pair(TRANS("Translation"), tmpStr));
 			type.push_back(PROPERTY_TYPE_POINT3D);
 			break;
 		}
+		case MODE_TRANSLATE_VALUE:
+		{
+			ASSERT(vectorParams.size() == 0);
+			ASSERT(scalarParams.size() == 1);
+			
+			
+			if(originMode == ORIGINMODE_SELECT)
+			{
+				stream_cast(tmpStr,scalarParams[0]);
+				keys.push_back(KEY_ORIGIN_VALUE);
+				s.push_back(make_pair(TRANS("Origin"), tmpStr));
+				type.push_back(PROPERTY_TYPE_REAL);
+			}
+			break;
+		}
 		case MODE_SCALE:
 		{
 			ASSERT(vectorParams.size() == 1);
 			ASSERT(scalarParams.size() == 1);
 			
-
+			
 			if(originMode == ORIGINMODE_SELECT)
 			{
 				stream_cast(tmpStr,vectorParams[0]);
 				keys.push_back(KEY_ORIGIN);
-				s.push_back(make_pair("Origin", tmpStr));
+				s.push_back(make_pair(TRANS("Origin"), tmpStr));
 				type.push_back(PROPERTY_TYPE_POINT3D);
 			}
-
+			
 			stream_cast(tmpStr,scalarParams[0]);
 			keys.push_back(KEY_SCALEFACTOR);
-			s.push_back(make_pair("Scale Fact.", tmpStr));
+			s.push_back(make_pair(TRANS("Scale Fact."), tmpStr));
 			type.push_back(PROPERTY_TYPE_REAL);
 			break;
 		}
@@ -977,17 +1121,17 @@ void TransformFilter::getProperties(FilterProperties &propertyList) const
 			{
 				stream_cast(tmpStr,vectorParams[0]);
 				keys.push_back(KEY_ORIGIN);
-				s.push_back(make_pair("Origin", tmpStr));
+				s.push_back(make_pair(TRANS("Origin"), tmpStr));
 				type.push_back(PROPERTY_TYPE_POINT3D);
 			}
 			stream_cast(tmpStr,vectorParams[1]);
 			keys.push_back(KEY_ROTATE_AXIS);
-			s.push_back(make_pair("Axis", tmpStr));
+			s.push_back(make_pair(TRANS("Axis"), tmpStr));
 			type.push_back(PROPERTY_TYPE_POINT3D);
 
 			stream_cast(tmpStr,scalarParams[0]);
 			keys.push_back(KEY_ROTATE_ANGLE);
-			s.push_back(make_pair("Angle (deg)", tmpStr));
+			s.push_back(make_pair(TRANS("Angle (deg)"), tmpStr));
 			type.push_back(PROPERTY_TYPE_REAL);
 			break;
 		}
@@ -1003,7 +1147,7 @@ void TransformFilter::getProperties(FilterProperties &propertyList) const
 			tmpStr= choiceString(choices,noiseType);
 			choices.clear();
 			
-			s.push_back(make_pair(string("Noise Type"),tmpStr));
+			s.push_back(make_pair(string(TRANS("Noise Type")),tmpStr));
 			type.push_back(PROPERTY_TYPE_CHOICE);
 			keys.push_back(KEY_NOISETYPE);
 
@@ -1011,9 +1155,9 @@ void TransformFilter::getProperties(FilterProperties &propertyList) const
 			stream_cast(tmpStr,scalarParams[0]);
 			keys.push_back(KEY_NOISELEVEL);
 			if(noiseType == NOISETYPE_WHITE)
-				s.push_back(make_pair("Noise level", tmpStr));
+				s.push_back(make_pair(TRANS("Noise level"), tmpStr));
 			else if(noiseType == NOISETYPE_GAUSSIAN)
-				s.push_back(make_pair("Standard dev.", tmpStr));
+				s.push_back(make_pair(TRANS("Standard dev."), tmpStr));
 			else
 			{
 				ASSERT(false);
@@ -1035,7 +1179,7 @@ void TransformFilter::getProperties(FilterProperties &propertyList) const
 		propertyList.data.push_back(s);
 		propertyList.types.push_back(type);
 		propertyList.keys.push_back(keys);
-		propertyList.keyNames.push_back("Transform Params");
+		propertyList.keyNames.push_back(TRANS("Transform Params"));
 	}
 
 }
@@ -1053,15 +1197,17 @@ bool TransformFilter::setProperty( unsigned int set, unsigned int key,
 			//TODO: Mkove these into an array,
 			//so they are synced with ::getProperties 
 			// (it wont work if these are not synced)
-			if(value == "Translate")
+			if(value == TRANS("Translate"))
 				transformMode= MODE_TRANSLATE;
-			else if ( value == "Scale" )
+			else if ( value == TRANS("Translate Value") )
+				transformMode= MODE_TRANSLATE_VALUE;
+			else if ( value == TRANS("Scale") )
 				transformMode= MODE_SCALE;
-			else if ( value == "Rotate")
+			else if ( value == TRANS("Rotate"))
 				transformMode= MODE_ROTATE;
-			else if ( value == "Value Shuffle")
+			else if ( value == TRANS("Value Shuffle"))
 				transformMode= MODE_VALUE_SHUFFLE;
-			else if ( value == "Spatial Noise")
+			else if ( value == TRANS("Spatial Noise"))
 				transformMode= MODE_SPATIAL_NOISE;
 			else
 				return false;
@@ -1077,6 +1223,9 @@ bool TransformFilter::setProperty( unsigned int set, unsigned int key,
 				case MODE_TRANSLATE:
 					vectorParams.push_back(Point3D(0,0,0));
 					break;
+				case MODE_TRANSLATE_VALUE:
+					scalarParams.push_back(100.0f);
+					break;
 				case MODE_ROTATE:
 					vectorParams.push_back(Point3D(0,0,0));
 					vectorParams.push_back(Point3D(1,0,0));
@@ -1100,6 +1249,7 @@ bool TransformFilter::setProperty( unsigned int set, unsigned int key,
 		case KEY_ROTATE_ANGLE:
 		case KEY_SCALEFACTOR:
 		case KEY_NOISELEVEL:
+		case KEY_ORIGIN_VALUE:
 		{
 			float newScale;
 			if(stream_cast(newScale,value))
@@ -1213,10 +1363,10 @@ std::string  TransformFilter::getErrString(unsigned int code) const
 	{
 		//User aborted in a callback
 		case ERR_CALLBACK_FAIL:
-			return std::string("Aborted");
+			return std::string(TRANS("Aborted"));
 		//Caught a memory issue
 		case ERR_NOMEM:
-			return std::string("Unable to allocate memory");
+			return std::string(TRANS("Unable to allocate memory"));
 	}
 }
 
@@ -1401,6 +1551,10 @@ bool TransformFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFil
 			if(vectorParams.size() != 2 || scalarParams.size() !=1)
 				return false;
 			break;
+		case MODE_TRANSLATE_VALUE:
+			if(vectorParams.size() != 0 || scalarParams.size() !=1)
+				return false;
+			break;
 		case MODE_VALUE_SHUFFLE:
 		case MODE_SPATIAL_NOISE:
 			break;
