@@ -2,34 +2,64 @@
 #include "voxelise.h"
 #include "../xmlHelper.h"
 
+#include "../translation.h"
+
 enum
 {
-	KEY_VOXEL_FIXEDWIDTH,
-	KEY_VOXEL_NBINSX,
-	KEY_VOXEL_NBINSY,
-	KEY_VOXEL_NBINSZ,
-	KEY_VOXEL_WIDTHBINSX,
-	KEY_VOXEL_WIDTHBINSY,
-	KEY_VOXEL_WIDTHBINSZ,
-	KEY_VOXEL_COUNT_TYPE,
-	KEY_VOXEL_NORMALISE_TYPE,
-	KEY_VOXEL_SPOTSIZE,
-	KEY_VOXEL_TRANSPARANCY,
-	KEY_VOXEL_COLOUR,
-	KEY_VOXEL_ISOLEVEL,
+	KEY_VOXELISE_FIXEDWIDTH,
+	KEY_VOXELISE_NBINSX,
+	KEY_VOXELISE_NBINSY,
+	KEY_VOXELISE_NBINSZ,
+	KEY_VOXELISE_WIDTHBINSX,
+	KEY_VOXELISE_WIDTHBINSY,
+	KEY_VOXELISE_WIDTHBINSZ,
+	KEY_VOXELISE_COUNT_TYPE,
+	KEY_VOXELISE_NORMALISE_TYPE,
+	KEY_VOXELISE_SPOTSIZE,
+	KEY_VOXELISE_TRANSPARANCY,
+	KEY_VOXELISE_COLOUR,
+	KEY_VOXELISE_ISOLEVEL,
 	KEY_VOXEL_REPRESENTATION_MODE,
-	KEY_VOXEL_ENABLE_NUMERATOR,
-	KEY_VOXEL_ENABLE_DENOMINATOR,
+	KEY_VOXELISE_FILTER_MODE,
+	KEY_VOXELISE_FILTER_BOUNDARY_MODE,
+	KEY_VOXELISE_FILTER_BINS,
+	KEY_VOXELISE_ENABLE_NUMERATOR,
+	KEY_VOXELISE_ENABLE_DENOMINATOR
 };
 
 //!Normalisation method
 enum
 {
-	VOXEL_NORMALISETYPE_NONE,// straight count
-	VOXEL_NORMALISETYPE_VOLUME,// density
-	VOXEL_NORMALISETYPE_ALLATOMSINVOXEL, // concentration
-	VOXEL_NORMALISETYPE_COUNT2INVOXEL,// ratio count1/count2
-	VOXEL_NORMALISETYPE_MAX, // keep this at the end so it's a bookend for the last value
+	VOXELISE_NORMALISETYPE_NONE,// straight count
+	VOXELISE_NORMALISETYPE_VOLUME,// density
+	VOXELISE_NORMALISETYPE_ALLATOMSINVOXEL, // concentration
+	VOXELISE_NORMALISETYPE_COUNT2INVOXEL,// ratio count1/count2
+	VOXELISE_NORMALISETYPE_MAX // keep this at the end so it's a bookend for the last value
+};
+
+//!Filtering mode
+enum
+{
+	VOXELISE_FILTERTYPE_NONE,
+	VOXELISE_FILTERTYPE_GAUSS,
+	VOXELISE_FILTERTYPE_MAX // keep this at the end so it's a bookend for the last value
+};
+
+
+//Boundary behaviour for filtering 
+enum
+{
+	VOXELISE_FILTERBOUNDMODE_ZERO,
+	VOXELISE_FILTERBOUNDMODE_BOUNCE,
+	VOXELISE_FILTERBOUNDMODE_MAX// keep this at the end so it's a bookend for the last value
+};
+
+enum
+{
+	VOXELISE_ABORT_ERR,
+	VOXELISE_MEMORY_ERR,
+	VOXELISE_CONVOLVE_ERR,
+	VOXELISE_BOUNDS_INVALID_ERR
 };
 
 //This is not a member of voxels.h, as the voxels do not have any concept of the IonHit
@@ -53,8 +83,7 @@ int countPoints(Voxels<float> &v, const std::vector<IonHit> &points,
 		v.getIndex(x,y,z,points[ui].getPos());
 
 		//Ensure it lies within the dataset
-		if (x < binCount[0] && y < binCount[1] && z< binCount[2]
-		        && x >= 0 && y >= 0 && z >= 0)
+		if (x < binCount[0] && y < binCount[1] && z< binCount[2])
 		{
 			{
 				float value;
@@ -75,21 +104,36 @@ int countPoints(Voxels<float> &v, const std::vector<IonHit> &points,
 
 // == Voxels filter ==
 VoxeliseFilter::VoxeliseFilter() 
-: fixedWidth(false), bc(), normaliseType(VOXEL_NORMALISETYPE_NONE)
+: fixedWidth(false), normaliseType(VOXELISE_NORMALISETYPE_NONE)
 {
 	splatSize=1.0f;
 	a=0.9f;
 	r=g=b=0.5;
 	isoLevel=0.5;
-	bc.setBounds(0, 0, 0, 1, 1, 1);
+	
+	filterBins=3;
+	filterMode=VOXELISE_FILTERTYPE_NONE;
+	filterBoundaryMode=VOXELISE_FILTERBOUNDMODE_BOUNCE;
+	gaussDev=0.5;	
+	
 	representation=VOXEL_REPRESENT_POINTCLOUD;
-	for (unsigned int i = 0; i < INDEX_LENGTH; i++) {
+
+
+	//Ficticious bounds.
+	bc.setBounds(Point3D(0,0,0),Point3D(1,1,1));
+
+	for (unsigned int i = 0; i < INDEX_LENGTH; i++) 
 		nBins[i] = 50;
-	}
-	calculateWidthsFromNumBins(binWidth, nBins);
+
+	calculateWidthsFromNumBins(binWidth,nBins);
+
 	numeratorAll = false;
 	denominatorAll = true;
 
+	cacheOK=false;
+	cache=true; //By default, we should cache, but decision is made higher up
+
+
 	rsdIncoming=0;
 }
 
@@ -102,11 +146,18 @@ Filter *VoxeliseFilter::cloneUncached() const
 	p->r=r;
 	p->g=g;
 	p->b=b;
-	
+
 	p->isoLevel=isoLevel;
+	
+	p->filterMode=filterMode;
+	p->filterBoundaryMode=filterBoundaryMode;
+	p->filterBins=filterBins;
+	p->gaussDev=gaussDev;
+
 	p->representation=representation;
 	p->splatSize=splatSize;
 
+	p->normaliseType=normaliseType;
 	p->numeratorAll=numeratorAll;
 	p->denominatorAll=denominatorAll;
 
@@ -132,6 +183,9 @@ Filter *VoxeliseFilter::cloneUncached() const
 	else
 		p->rsdIncoming=0;
 
+	p->cache=cache;
+	p->cacheOK=false;
+	p->userString=userString;
 	return p;
 }
 
@@ -212,10 +266,25 @@ void VoxeliseFilter::initFilter(const std::vector<const FilterStreamData *> &dat
 	}
 }
 
-// TODO: create plotstream
+//TODO:
 unsigned int VoxeliseFilter::refresh(const std::vector<const FilterStreamData *> &dataIn,
 										  std::vector<const FilterStreamData *> &getOut, ProgressData &progress, bool (*callback)(void))
-{	
+{
+	for(size_t ui=0;ui<dataIn.size();ui++)
+	{
+		//Disallow copying of anything in the blockmask. Copy everything else
+		if(!(dataIn[ui]->getStreamType() & getRefreshBlockMask() ))
+			getOut.push_back(dataIn[ui]);
+	}
+	
+	//use the cached copy if we have it.
+	if(cacheOK)
+	{
+		for(size_t ui=0;ui<filterOutputs.size();ui++)
+			getOut.push_back(filterOutputs[ui]);
+		return 0;
+	}
+
 
 	Point3D minP,maxP;
 
@@ -239,7 +308,7 @@ unsigned int VoxeliseFilter::refresh(const std::vector<const FilterStreamData *>
 		bc.expand(bcTmp);
 	}
 	//No bounding box? Tough cookies
-	if (!bc.isValid() || bc.isFlat()) return VOXEL_BOUNDS_INVALID_ERR;
+	if (!bc.isValid() || bc.isFlat()) return VOXELISE_BOUNDS_INVALID_ERR;
 
 	bc.getBounds(minP,maxP);	
 	if (fixedWidth) 
@@ -252,7 +321,7 @@ unsigned int VoxeliseFilter::refresh(const std::vector<const FilterStreamData *>
 		return 0;
 		
 	VoxelStreamData *vs = new VoxelStreamData();
-	vs->cached = 0;
+	vs->parent=this;
 	vs->data.setCallbackMethod(callback);
 	vs->data.init(nBins[0], nBins[1], nBins[2], bc);
 	vs->representationType= representation;
@@ -264,20 +333,14 @@ unsigned int VoxeliseFilter::refresh(const std::vector<const FilterStreamData *>
 	vs->b=b;
 	vs->a=a;
 
-	VoxelStreamData *vsDenom = NULL;
-	if (normaliseType == VOXEL_NORMALISETYPE_COUNT2INVOXEL ||
-		normaliseType == VOXEL_NORMALISETYPE_ALLATOMSINVOXEL) {
+	Voxels<float> vsDenom;
+	if (normaliseType == VOXELISE_NORMALISETYPE_COUNT2INVOXEL ||
+		normaliseType == VOXELISE_NORMALISETYPE_ALLATOMSINVOXEL) {
 		//Check we actually have incoming data
 		ASSERT(rsdIncoming);
-		vsDenom = new VoxelStreamData();
-		vsDenom->cached = 0;
-		vsDenom->data.setCallbackMethod(callback);
-		vsDenom->data.init(nBins[0], nBins[1], nBins[2], bc);
-		vsDenom->representationType= representation;
-		vsDenom->splatSize = splatSize;
-		vsDenom->isoLevel=isoLevel;
-		vsDenom->data.fill(0);
-		vsDenom->a=a;
+		vsDenom.setCallbackMethod(callback);
+		vsDenom.init(nBins[0], nBins[1], nBins[2], bc);
+		vsDenom.fill(0);
 	}
 
 	const IonStreamData *is;
@@ -314,7 +377,7 @@ unsigned int VoxeliseFilter::refresh(const std::vector<const FilterStreamData *>
 			}
 		
 			//If the user requests normalisation, compute the denominator datset
-			if (normaliseType == VOXEL_NORMALISETYPE_COUNT2INVOXEL) {
+			if (normaliseType == VOXELISE_NORMALISETYPE_COUNT2INVOXEL) {
 				if(is->data.size())
 				{
 					//Check what Ion type this stream belongs to. Assume all ions
@@ -329,26 +392,26 @@ unsigned int VoxeliseFilter::refresh(const std::vector<const FilterStreamData *>
 						thisIonEnabled=false;
 
 					if(thisIonEnabled)
-						countPoints(vsDenom->data,is->data,true,callback);
+						countPoints(vsDenom,is->data,true,callback);
 				}
-			} else if (normaliseType == VOXEL_NORMALISETYPE_ALLATOMSINVOXEL)
+			} else if (normaliseType == VOXELISE_NORMALISETYPE_ALLATOMSINVOXEL)
 			{
-				countPoints(vsDenom->data,is->data,true,callback);
+				countPoints(vsDenom,is->data,true,callback);
 			}
 
 			if(!(*callback)())
 			{
 				delete vs;
-				return VOXEL_ABORT_ERR;
+				return VOXELISE_ABORT_ERR;
 			}
 		}
 	
 		//Perform normalsiation	
-		if (normaliseType == VOXEL_NORMALISETYPE_VOLUME)
+		if (normaliseType == VOXELISE_NORMALISETYPE_VOLUME)
 			vs->data.calculateDensity();
-		else if (normaliseType == VOXEL_NORMALISETYPE_COUNT2INVOXEL ||
-				 normaliseType == VOXEL_NORMALISETYPE_ALLATOMSINVOXEL)
-			vs->data /= vsDenom->data;
+		else if (normaliseType == VOXELISE_NORMALISETYPE_COUNT2INVOXEL ||
+				 normaliseType == VOXELISE_NORMALISETYPE_ALLATOMSINVOXEL)
+			vs->data /= vsDenom;
 	}
 	else
 	{
@@ -365,19 +428,61 @@ unsigned int VoxeliseFilter::refresh(const std::vector<const FilterStreamData *>
 				if(!(*callback)())
 				{
 					delete vs;
-					return VOXEL_ABORT_ERR;
+					return VOXELISE_ABORT_ERR;
 				}
 
 			}
 		}
-		ASSERT(normaliseType != VOXEL_NORMALISETYPE_COUNT2INVOXEL
-				&& normaliseType!=VOXEL_NORMALISETYPE_ALLATOMSINVOXEL);
-		if (normaliseType == VOXEL_NORMALISETYPE_VOLUME)
+		ASSERT(normaliseType != VOXELISE_NORMALISETYPE_COUNT2INVOXEL
+				&& normaliseType!=VOXELISE_NORMALISETYPE_ALLATOMSINVOXEL);
+		if (normaliseType == VOXELISE_NORMALISETYPE_VOLUME)
 			vs->data.calculateDensity();
 	}	
-	delete vsDenom;
 
-	
+	vsDenom.clear();
+
+
+	//Perform voxel filtering
+	switch(filterMode)
+	{
+		case VOXELISE_FILTERTYPE_NONE:
+			break;
+		case VOXELISE_FILTERTYPE_GAUSS:
+		{
+			Voxels<float> kernel,res;
+
+			map<unsigned int, unsigned int> modeMap;
+
+
+			modeMap[VOXELISE_FILTERBOUNDMODE_ZERO]=BOUND_ZERO;
+			modeMap[VOXELISE_FILTERBOUNDMODE_BOUNCE]=BOUND_MIRROR;
+
+			//FIXME: This will be SLOW. need to use IIR or some other
+			//fast technique
+			
+			//Construct the gaussian convolution
+			kernel.setGaussianKernelCube(gaussDev,(float)filterBins,filterBins);
+			//Normalise the kernel
+			float sum;
+			sum=kernel.getSum();
+			kernel/=sum;
+		
+			if(res.resize(vs->data))
+				return VOXELISE_MEMORY_ERR; 
+
+			//Gaussian kernel is separable (rank 1)
+			if(vs->data.separableConvolve(kernel,res,false,modeMap[filterBoundaryMode]))
+				return VOXELISE_CONVOLVE_ERR;
+
+			vs->data.swap(res);
+
+			res.clear();
+			break;
+		}
+		default:
+			ASSERT(false);
+	}
+
 	
 	float min,max;
 	vs->data.minMax(min,max);
@@ -386,44 +491,75 @@ unsigned int VoxeliseFilter::refresh(const std::vector<const FilterStreamData *>
 	string sMin,sMax;
 	stream_cast(sMin,min);
 	stream_cast(sMax,max);
-	consoleOutput.push_back(std::string("Voxel Limits (min,max): (") + sMin + string(",")
+	consoleOutput.push_back(std::string(TRANS("Voxel Limits (min,max): (") + sMin + string(","))
 		       	+  sMax + ")");
-	getOut.push_back(vs);
+	if(cache)
+	{
+		vs->cached=1;
+		cacheOK=true;
+		filterOutputs.push_back(vs);
+	}
+	else
+		vs->cached=0;
 
+
+	//Store the voxels on the output
+	getOut.push_back(vs);
+	
+	//Copy the inputs into the outputs, provided they are not voxels
 	return 0;
 }
 
 std::string VoxeliseFilter::getNormaliseTypeString(int type) const {
 	switch (type) {
-		case VOXEL_NORMALISETYPE_NONE:
-			return std::string("None (Raw count)");
-			break;
-		case VOXEL_NORMALISETYPE_VOLUME:
-			return std::string("Volume (Density)");
-			break;
-		case VOXEL_NORMALISETYPE_COUNT2INVOXEL:
-			return std::string("Ratio (Num/Denom)");
-			break;
-		case VOXEL_NORMALISETYPE_ALLATOMSINVOXEL:
-			return std::string("All Ions (conc)");
-			break;
+		case VOXELISE_NORMALISETYPE_NONE:
+			return std::string(TRANS("None (Raw count)"));
+		case VOXELISE_NORMALISETYPE_VOLUME:
+			return std::string(TRANS("Volume (Density)"));
+		case VOXELISE_NORMALISETYPE_COUNT2INVOXEL:
+			return std::string(TRANS("Ratio (Num/Denom)"));
+		case VOXELISE_NORMALISETYPE_ALLATOMSINVOXEL:
+			return std::string(TRANS("All Ions (conc)"));
 		default:
-			return "";
-			break;
+			ASSERT(false);
 	}
 }
 
 std::string VoxeliseFilter::getRepresentTypeString(int type) const {
 	switch (type) {
 		case VOXEL_REPRESENT_POINTCLOUD:
-			return std::string("Point Cloud");
-			break;
+			return std::string(TRANS("Point Cloud"));
 		case VOXEL_REPRESENT_ISOSURF:
-			return std::string("Isosurface");
-			break;
+			return std::string(TRANS("Isosurface"));
 		default:
-			return "";
-			break;
+			ASSERT(false);
+	}
+}
+
+std::string VoxeliseFilter::getFilterTypeString(int type) const 
+{
+	switch (type)
+	{
+		case VOXELISE_FILTERTYPE_NONE:
+			return std::string(TRANS("None"));
+		case VOXELISE_FILTERTYPE_GAUSS:
+			return std::string(TRANS("Gaussian (2𝜎)"));
+		default:
+			ASSERT(false);
+	}
+}
+
+
+std::string VoxeliseFilter::getFilterBoundTypeString(int type) const 
+{
+	switch (type)
+	{
+		case VOXELISE_FILTERBOUNDMODE_ZERO:
+			return std::string(TRANS("Zero"));
+		case VOXELISE_FILTERBOUNDMODE_BOUNCE:
+			return std::string(TRANS("Bounce"));
+		default:
+			ASSERT(false);
 	}
 }
 
@@ -438,71 +574,67 @@ void VoxeliseFilter::getProperties(FilterProperties &propertyList) const
 
 	string tmpStr;
 	stream_cast(tmpStr, fixedWidth);
-	s.push_back(std::make_pair("Fixed width", tmpStr));
-	keys.push_back(KEY_VOXEL_FIXEDWIDTH);
+	s.push_back(std::make_pair(TRANS("Fixed width"), tmpStr));
+	keys.push_back(KEY_VOXELISE_FIXEDWIDTH);
 	type.push_back(PROPERTY_TYPE_BOOL);
 	
 	if(fixedWidth)
 	{
 		stream_cast(tmpStr,binWidth[0]);
-		keys.push_back(KEY_VOXEL_WIDTHBINSX);
-		s.push_back(make_pair("Bin width x", tmpStr));
+		keys.push_back(KEY_VOXELISE_WIDTHBINSX);
+		s.push_back(make_pair(TRANS("Bin width x"), tmpStr));
 		type.push_back(PROPERTY_TYPE_REAL);
 
 		stream_cast(tmpStr,binWidth[1]);
-		keys.push_back(KEY_VOXEL_WIDTHBINSY);
-		s.push_back(make_pair("Bin width y", tmpStr));
+		keys.push_back(KEY_VOXELISE_WIDTHBINSY);
+		s.push_back(make_pair(TRANS("Bin width y"), tmpStr));
 		type.push_back(PROPERTY_TYPE_REAL);
 
 		stream_cast(tmpStr,binWidth[2]);
-		keys.push_back(KEY_VOXEL_WIDTHBINSZ);
-		s.push_back(make_pair("Bin width z", tmpStr));
+		keys.push_back(KEY_VOXELISE_WIDTHBINSZ);
+		s.push_back(make_pair(TRANS("Bin width z"), tmpStr));
 		type.push_back(PROPERTY_TYPE_REAL);
 	}
 	else
 	{
 		stream_cast(tmpStr,nBins[0]);
-		keys.push_back(KEY_VOXEL_NBINSX);
-		s.push_back(make_pair("Num bins x", tmpStr));
+		keys.push_back(KEY_VOXELISE_NBINSX);
+		s.push_back(make_pair(TRANS("Num bins x"), tmpStr));
 		type.push_back(PROPERTY_TYPE_INTEGER);
 		
 		stream_cast(tmpStr,nBins[1]);
-		keys.push_back(KEY_VOXEL_NBINSY);
-		s.push_back(make_pair("Num bins y", tmpStr));
+		keys.push_back(KEY_VOXELISE_NBINSY);
+		s.push_back(make_pair(TRANS("Num bins y"), tmpStr));
 		type.push_back(PROPERTY_TYPE_INTEGER);
 		
 		stream_cast(tmpStr,nBins[2]);
-		keys.push_back(KEY_VOXEL_NBINSZ);
-		s.push_back(make_pair("Num bins z", tmpStr));
+		keys.push_back(KEY_VOXELISE_NBINSZ);
+		s.push_back(make_pair(TRANS("Num bins z"), tmpStr));
 		type.push_back(PROPERTY_TYPE_INTEGER);
 	}
 
 	//Let the user know what the valid values for voxel value types are
 	string tmpChoice;	
 	vector<pair<unsigned int,string> > choices;
-	tmpStr=getNormaliseTypeString(VOXEL_NORMALISETYPE_NONE);
-	choices.push_back(make_pair((unsigned int)VOXEL_NORMALISETYPE_NONE,tmpStr));
-	tmpStr=getNormaliseTypeString(VOXEL_NORMALISETYPE_VOLUME);
-	choices.push_back(make_pair((unsigned int)VOXEL_NORMALISETYPE_VOLUME,tmpStr));
+	tmpStr=getNormaliseTypeString(VOXELISE_NORMALISETYPE_NONE);
+	choices.push_back(make_pair((unsigned int)VOXELISE_NORMALISETYPE_NONE,tmpStr));
+	tmpStr=getNormaliseTypeString(VOXELISE_NORMALISETYPE_VOLUME);
+	choices.push_back(make_pair((unsigned int)VOXELISE_NORMALISETYPE_VOLUME,tmpStr));
 	if(rsdIncoming)
 	{
 		//Concentration mode
-		tmpStr=getNormaliseTypeString(VOXEL_NORMALISETYPE_ALLATOMSINVOXEL);
-		choices.push_back(make_pair((unsigned int)VOXEL_NORMALISETYPE_ALLATOMSINVOXEL,tmpStr));
-		//Ratio is only valid if we have a way of seperation for the ions i.e. range
-		tmpStr=getNormaliseTypeString(VOXEL_NORMALISETYPE_COUNT2INVOXEL);
-		choices.push_back(make_pair((unsigned int)VOXEL_NORMALISETYPE_COUNT2INVOXEL,tmpStr));
+		tmpStr=getNormaliseTypeString(VOXELISE_NORMALISETYPE_ALLATOMSINVOXEL);
+		choices.push_back(make_pair((unsigned int)VOXELISE_NORMALISETYPE_ALLATOMSINVOXEL,tmpStr));
+		//Ratio is only valid if we have a way of separation for the ions i.e. range
+		tmpStr=getNormaliseTypeString(VOXELISE_NORMALISETYPE_COUNT2INVOXEL);
+		choices.push_back(make_pair((unsigned int)VOXELISE_NORMALISETYPE_COUNT2INVOXEL,tmpStr));
 	}
 	tmpStr= choiceString(choices,normaliseType);
-	s.push_back(make_pair(string("Normalise by"),tmpStr));
+	s.push_back(make_pair(string(TRANS("Normalise by")),tmpStr));
 	type.push_back(PROPERTY_TYPE_CHOICE);
-	keys.push_back(KEY_VOXEL_NORMALISE_TYPE);
+	keys.push_back(KEY_VOXELISE_NORMALISE_TYPE);
 	
 	
-	//TODO
-	//1. range file
-	//2. threshold
-	//3. gaussian
 	
 	propertyList.data.push_back(s);
 	propertyList.types.push_back(type);
@@ -515,9 +647,9 @@ void VoxeliseFilter::getProperties(FilterProperties &propertyList) const
 		
 	// numerator
 	if (rsdIncoming) {
-		s.push_back(make_pair("Numerator", numeratorAll ? "1" : "0"));
+		s.push_back(make_pair(TRANS("Numerator"), numeratorAll ? "1" : "0"));
 		type.push_back(PROPERTY_TYPE_BOOL);
-		keys.push_back(KEY_VOXEL_ENABLE_NUMERATOR);
+		keys.push_back(KEY_VOXELISE_ENABLE_NUMERATOR);
 
 		ASSERT(rsdIncoming->enabledIons.size()==enabledIons[0].size());	
 		ASSERT(rsdIncoming->enabledIons.size()==enabledIons[1].size());	
@@ -535,7 +667,7 @@ void VoxeliseFilter::getProperties(FilterProperties &propertyList) const
 			s.push_back(make_pair(
 				rsdIncoming->rangeFile->getName(ui), str));
 			type.push_back(PROPERTY_TYPE_BOOL);
-			keys.push_back(KEY_VOXEL_ENABLE_NUMERATOR*1000+ui);
+			keys.push_back(KEY_VOXELISE_ENABLE_NUMERATOR*1000+ui);
 		}
 		propertyList.types.push_back(type);
 		propertyList.data.push_back(s);
@@ -546,11 +678,11 @@ void VoxeliseFilter::getProperties(FilterProperties &propertyList) const
 	type.clear();
 	keys.clear();
 	
-	if (normaliseType == VOXEL_NORMALISETYPE_COUNT2INVOXEL && rsdIncoming) {
+	if (normaliseType == VOXELISE_NORMALISETYPE_COUNT2INVOXEL && rsdIncoming) {
 		// denominator
-		s.push_back(make_pair("Denominator", denominatorAll ? "1" : "0"));
+		s.push_back(make_pair(TRANS("Denominator"), denominatorAll ? "1" : "0"));
 		type.push_back(PROPERTY_TYPE_BOOL);
-		keys.push_back(KEY_VOXEL_ENABLE_DENOMINATOR);
+		keys.push_back(KEY_VOXELISE_ENABLE_DENOMINATOR);
 
 		for(unsigned  int ui=0; ui<rsdIncoming->enabledIons.size(); ui++)
 		{			
@@ -565,7 +697,7 @@ void VoxeliseFilter::getProperties(FilterProperties &propertyList) const
 				rsdIncoming->rangeFile->getName(ui), str));
 
 			type.push_back(PROPERTY_TYPE_BOOL);
-			keys.push_back(KEY_VOXEL_ENABLE_DENOMINATOR*1000+ui);
+			keys.push_back(KEY_VOXELISE_ENABLE_DENOMINATOR*1000+ui);
 		}
 		propertyList.types.push_back(type);
 		propertyList.data.push_back(s);
@@ -575,7 +707,59 @@ void VoxeliseFilter::getProperties(FilterProperties &propertyList) const
 		type.clear();
 		keys.clear();
 	}
+
+	//Start a new set for filtering
+	//----
+	//TODO: Other filtering? threshold/median? laplacian? etc
 	
+	choices.clear();
+
+	//Post-filtering method
+	for(unsigned int ui=0;ui<VOXELISE_FILTERTYPE_MAX; ui++)
+	{
+		tmpStr=getFilterTypeString(ui);
+		choices.push_back(make_pair(ui,tmpStr));
+	}
+	tmpStr= choiceString(choices,filterMode);
+	s.push_back(make_pair(TRANS("Filtering"), tmpStr));
+	type.push_back(PROPERTY_TYPE_CHOICE);
+	keys.push_back(KEY_VOXELISE_FILTER_MODE);
+	
+	if(filterMode != VOXELISE_FILTERTYPE_NONE)
+	{
+
+		//Filter size
+		stream_cast(tmpStr,filterBins);
+		s.push_back(make_pair(TRANS("Kernel Bins"), tmpStr));
+		type.push_back(PROPERTY_TYPE_INTEGER);
+		keys.push_back(KEY_VOXELISE_FILTER_BINS);
+
+		//Boundary wrapping mode selection
+		choices.clear();
+		for(unsigned int ui=0;ui<VOXELISE_FILTERBOUNDMODE_MAX; ui++)
+		{
+			tmpStr=getFilterBoundTypeString(ui);
+			choices.push_back(make_pair(ui,tmpStr));
+		}
+		tmpStr= choiceString(choices,filterBoundaryMode);
+	
+		s.push_back(make_pair(TRANS("Exterior values"), tmpStr));
+		type.push_back(PROPERTY_TYPE_CHOICE);
+		keys.push_back(KEY_VOXELISE_FILTER_MODE);
+
+	}
+
+
+	propertyList.types.push_back(type);
+	propertyList.data.push_back(s);
+	propertyList.keys.push_back(keys);
+	
+
+	s.clear();
+	type.clear();
+	keys.clear();
+	//----
+
 	//start a new set for the visual representation
 	//----------------------------
 	choices.clear();
@@ -586,39 +770,45 @@ void VoxeliseFilter::getProperties(FilterProperties &propertyList) const
 	
 	
 	tmpStr= choiceString(choices,representation);
-	s.push_back(make_pair(string("Representation"),tmpStr));
+	s.push_back(make_pair(string(TRANS("Representation")),tmpStr));
 	type.push_back(PROPERTY_TYPE_CHOICE);
 	keys.push_back(KEY_VOXEL_REPRESENTATION_MODE);
+
 	switch(representation)
 	{
 		case VOXEL_REPRESENT_POINTCLOUD:
 		{
 			stream_cast(tmpStr,splatSize);
-			s.push_back(make_pair("Spot size",tmpStr));
+			s.push_back(make_pair(TRANS("Spot size"),tmpStr));
 			type.push_back(PROPERTY_TYPE_REAL);
-			keys.push_back(KEY_VOXEL_SPOTSIZE);
+			keys.push_back(KEY_VOXELISE_SPOTSIZE);
 
 			stream_cast(tmpStr,1.0-a);
-			s.push_back(make_pair("Transparency",tmpStr));
+			s.push_back(make_pair(TRANS("Transparency"),tmpStr));
 			type.push_back(PROPERTY_TYPE_REAL);
-			keys.push_back(KEY_VOXEL_TRANSPARANCY);
+			keys.push_back(KEY_VOXELISE_TRANSPARANCY);
 			break;
 		}
 		case VOXEL_REPRESENT_ISOSURF:
 		{
 			stream_cast(tmpStr,isoLevel);
-			s.push_back(make_pair("Isovalue",tmpStr));
+			s.push_back(make_pair(TRANS("Isovalue"),tmpStr));
 			type.push_back(PROPERTY_TYPE_REAL);
-			keys.push_back(KEY_VOXEL_ISOLEVEL);
+			keys.push_back(KEY_VOXELISE_ISOLEVEL);
 		
 				
 
 			//Convert the ion colour to a hex string	
 			genColString((unsigned char)(r*255),(unsigned char)(g*255),
 					(unsigned char)(b*255),(unsigned char)(a*255),tmpStr);
-			s.push_back(make_pair("Colour",tmpStr));
+			s.push_back(make_pair(TRANS("Colour"),tmpStr));
 			type.push_back(PROPERTY_TYPE_COLOUR);
-			keys.push_back(KEY_VOXEL_COLOUR);
+			keys.push_back(KEY_VOXELISE_COLOUR);
+
+			stream_cast(tmpStr,1.0-a);
+			s.push_back(make_pair(TRANS("Transparency"),tmpStr));
+			type.push_back(PROPERTY_TYPE_REAL);
+			keys.push_back(KEY_VOXELISE_TRANSPARANCY);
 			
 			break;
 		}
@@ -641,123 +831,199 @@ bool VoxeliseFilter::setProperty( unsigned int set, unsigned int key,
 	needUpdate=false;
 	switch(key)
 	{
-		case KEY_VOXEL_FIXEDWIDTH: 
+		case KEY_VOXELISE_FIXEDWIDTH: 
 		{
 			bool b;
 			if(stream_cast(b,value))
 				return false;
-			fixedWidth=b;
-			needUpdate=true;			
+
+			//if the result is different, the
+			//cache should be invalidated
+			if(b!=fixedWidth)
+			{
+				needUpdate=true;
+				fixedWidth=b;
+				clearCache();
+			}
 			break;
 		}	
-		case KEY_VOXEL_NBINSX:
+		case KEY_VOXELISE_NBINSX:
 		{
-			int i;
+			 unsigned int i;
 			if(stream_cast(i,value))
 				return false;
 			if(i <= 0)
 				return false;
+
 			needUpdate=true;
 			nBins[0]=i;
-			calculateWidthsFromNumBins(binWidth, nBins);
+			//if the result is different, the
+			//cache should be invalidated
+			if(i!=nBins[0])
+			{
+				needUpdate=true;
+				nBins[0]=i;
+				calculateWidthsFromNumBins(binWidth, nBins);
+				clearCache();
+			}
 			break;
 		}
-		case KEY_VOXEL_NBINSY:
+		case KEY_VOXELISE_NBINSY:
 		{
-			int i;
+			 unsigned int i;
 			if(stream_cast(i,value))
 				return false;
 			if(i <= 0)
 				return false;
 			needUpdate=true;
-			nBins[1]=i;
-			calculateWidthsFromNumBins(binWidth, nBins);
+			//if the result is different, the
+			//cache should be invalidated
+			if(i!=nBins[1])
+			{
+				needUpdate=true;
+				nBins[1]=i;
+				calculateWidthsFromNumBins(binWidth, nBins);
+				clearCache();
+			}
 			break;
 		}
-		case KEY_VOXEL_NBINSZ:
+		case KEY_VOXELISE_NBINSZ:
 		{
-			int i;
+			 unsigned int i;
 			if(stream_cast(i,value))
 				return false;
 			if(i <= 0)
 				return false;
-			needUpdate=true;
-			nBins[2]=i;
-			calculateWidthsFromNumBins(binWidth, nBins);
+			
+			//if the result is different, the
+			//cache should be invalidated
+			if(i!=nBins[2])
+			{
+				needUpdate=true;
+				nBins[2]=i;
+				calculateWidthsFromNumBins(binWidth, nBins);
+				clearCache();
+			}
 			break;
 		}
-		case KEY_VOXEL_WIDTHBINSX:
+		case KEY_VOXELISE_WIDTHBINSX:
 		{
 			float f;
 			if(stream_cast(f,value))
 				return false;
 			if(f <= 0.0f)
 				return false;
-			needUpdate=true;
-			binWidth[0]=f;
-			calculateNumBinsFromWidths(binWidth, nBins);
+		
+			if(f!=binWidth[0])
+			{
+				needUpdate=true;
+				binWidth[0]=f;
+				calculateNumBinsFromWidths(binWidth, nBins);
+				clearCache();
+			}
 			break;
 		}
-		case KEY_VOXEL_WIDTHBINSY:
+		case KEY_VOXELISE_WIDTHBINSY:
 		{
 			float f;
 			if(stream_cast(f,value))
 				return false;
 			if(f <= 0.0f)
 				return false;
-			needUpdate=true;
-			binWidth[1]=f;
-			calculateNumBinsFromWidths(binWidth, nBins);
+			
+			if(f!=binWidth[1])
+			{
+				needUpdate=true;
+				binWidth[1]=f;
+				calculateNumBinsFromWidths(binWidth, nBins);
+				clearCache();
+			}
 			break;
 		}
-		case KEY_VOXEL_WIDTHBINSZ:
+		case KEY_VOXELISE_WIDTHBINSZ:
 		{
 			float f;
 			if(stream_cast(f,value))
 				return false;
 			if(f <= 0.0f)
 				return false;
-			needUpdate=true;
-			binWidth[2]=f;
-			calculateNumBinsFromWidths(binWidth, nBins);
+			if(f!=binWidth[2])
+			{
+				needUpdate=true;
+				binWidth[2]=f;
+				calculateNumBinsFromWidths(binWidth, nBins);
+				clearCache();
+			}
 			break;
 		}
-		case KEY_VOXEL_NORMALISE_TYPE:
+		case KEY_VOXELISE_NORMALISE_TYPE:
 		{
-			int i;
-			for (i = 0; i < VOXEL_NORMALISETYPE_MAX; i++)
+			 unsigned int i;
+			for (i = 0; i < VOXELISE_NORMALISETYPE_MAX; i++)
 				if (value == getNormaliseTypeString(i)) break;
-			if (i == VOXEL_NORMALISETYPE_MAX)
+			if (i == VOXELISE_NORMALISETYPE_MAX)
 				return false;
 			if(normaliseType!=i)
+			{
 				needUpdate=true;
-			normaliseType=i;
+				clearCache();
+				normaliseType=i;
+			}
 			break;
 		}
-		case KEY_VOXEL_SPOTSIZE:
+		case KEY_VOXELISE_SPOTSIZE:
 		{
 			float f;
 			if(stream_cast(f,value))
 				return false;
 			if(f <= 0.0f)
 				return false;
-			needUpdate=true;
-			splatSize=f;
+			if(f !=splatSize)
+			{
+				splatSize=f;
+				needUpdate=true;
+
+				//Go in and manually adjust the cached
+				//entries to have the new value, rather
+				//than doing a full recomputation
+				if(cacheOK)
+				{
+					for(unsigned int ui=0;ui<filterOutputs.size();ui++)
+					{
+						VoxelStreamData *d;
+						d=(VoxelStreamData*)filterOutputs[ui];
+						d->splatSize=splatSize;
+					}
+				}
+
+			}
 			break;
 		}
-		case KEY_VOXEL_TRANSPARANCY:
+		case KEY_VOXELISE_TRANSPARANCY:
 		{
 			float f;
 			if(stream_cast(f,value))
 				return false;
-			if(f <= 0.0f || f > 1.0)
+			if(f < 0.0f || f > 1.0)
 				return false;
 			needUpdate=true;
 			//Alpha is opacity, which is 1-transparancy
 			a=1.0f-f;
+			//Go in and manually adjust the cached
+			//entries to have the new value, rather
+			//than doing a full recomputation
+			if(cacheOK)
+			{
+				for(unsigned int ui=0;ui<filterOutputs.size();ui++)
+				{
+					VoxelStreamData *d;
+					d=(VoxelStreamData*)filterOutputs[ui];
+					d->a=a;
+				}
+			}
 			break;
 		}
-		case KEY_VOXEL_ISOLEVEL:
+		case KEY_VOXELISE_ISOLEVEL:
 		{
 			float f;
 			if(stream_cast(f,value))
@@ -766,9 +1032,21 @@ bool VoxeliseFilter::setProperty( unsigned int set, unsigned int key,
 				return false;
 			needUpdate=true;
 			isoLevel=f;
+			//Go in and manually adjust the cached
+			//entries to have the new value, rather
+			//than doing a full recomputation
+			if(cacheOK)
+			{
+				for(unsigned int ui=0;ui<filterOutputs.size();ui++)
+				{
+					VoxelStreamData *d;
+					d=(VoxelStreamData*)filterOutputs[ui];
+					d->isoLevel=isoLevel;
+				}
+			}
 			break;
 		}
-		case KEY_VOXEL_COLOUR:
+		case KEY_VOXELISE_COLOUR:
 		{
 			unsigned char newR,newG,newB,newA;
 
@@ -780,20 +1058,46 @@ bool VoxeliseFilter::setProperty( unsigned int set, unsigned int key,
 			r=newR/255.0;
 			g=newG/255.0;
 			b=newB/255.0;
+			//Go in and manually adjust the cached
+			//entries to have the new value, rather
+			//than doing a full recomputation
+			if(cacheOK)
+			{
+				for(unsigned int ui=0;ui<filterOutputs.size();ui++)
+				{
+					VoxelStreamData *d;
+					d=(VoxelStreamData*)filterOutputs[ui];
+					d->r=r;
+					d->g=g;
+					d->b=b;
+				}
+			}
 			break;
 		}
 		case KEY_VOXEL_REPRESENTATION_MODE:
 		{
-			int i;
+			 unsigned int i;
 			for (i = 0; i < VOXEL_REPRESENT_END; i++)
 				if (value == getRepresentTypeString(i)) break;
 			if (i == VOXEL_REPRESENT_END)
 				return false;
 			needUpdate=true;
 			representation=i;
+			//Go in and manually adjust the cached
+			//entries to have the new value, rather
+			//than doing a full recomputation
+			if(cacheOK)
+			{
+				for(unsigned int ui=0;ui<filterOutputs.size();ui++)
+				{
+					VoxelStreamData *d;
+					d=(VoxelStreamData*)filterOutputs[ui];
+					d->representationType=representation;
+				}
+			}
 			break;
 		}
-		case KEY_VOXEL_ENABLE_NUMERATOR:
+		case KEY_VOXELISE_ENABLE_NUMERATOR:
 		{
 			bool b;
 			if(stream_cast(b,value))
@@ -802,10 +1106,11 @@ bool VoxeliseFilter::setProperty( unsigned int set, unsigned int key,
 			for (size_t i = 0; i < enabledIons[0].size(); i++) 
 				enabledIons[0][i] = b;
 			numeratorAll = b;
-			needUpdate=true;			
+			needUpdate=true;
+			clearCache();
 			break;
 		}
-		case KEY_VOXEL_ENABLE_DENOMINATOR:
+		case KEY_VOXELISE_ENABLE_DENOMINATOR:
 		{
 			bool b;
 			if(stream_cast(b,value))
@@ -817,34 +1122,85 @@ bool VoxeliseFilter::setProperty( unsigned int set, unsigned int key,
 			
 			denominatorAll = b;
 			needUpdate=true;			
+			clearCache();
+			break;
+		}
+		case KEY_VOXELISE_FILTER_MODE:
+		{
+			 unsigned int i;
+			for (i = 0; i < VOXEL_REPRESENT_END; i++)
+				if (value == getFilterTypeString(i)) break;
+			if (i == VOXEL_REPRESENT_END)
+				return false;
+			if(i!=filterMode)
+			{
+				needUpdate=true;
+				filterMode=i;
+				clearCache();
+			}
+			break;
+		}
+		case KEY_VOXELISE_FILTER_BOUNDARY_MODE:
+		{
+			 unsigned int i;
+			for (i = 0; i < VOXELISE_FILTERBOUNDMODE_MAX; i++)
+				if (value == getFilterBoundTypeString(i)) break;
+			if (i == VOXELISE_FILTERTYPE_MAX)
+				return false;
+			
+			if(i != filterBoundaryMode)
+			{
+				filterBoundaryMode=i;
+				needUpdate=true;
+				clearCache();
+			}
+			break;
+		}
+		case KEY_VOXELISE_FILTER_BINS:
+		{
+			 unsigned int i;
+			if(stream_cast(i,value))
+				return false;
+
+			//FIXME: Min restriction is artificial and imposed due to incomplete separerable convolution filter implementation
+			if(i <= 0 || i > std::min(nBins[0],std::min(nBins[1],nBins[2])))
+				return false;
+			if(i != filterBins)
+			{
+				needUpdate=true;
+				filterBins=i;
+				clearCache();
+			}
 			break;
 		}
 		default:
 		{
-			if (key >= KEY_VOXEL_ENABLE_DENOMINATOR*1000) {
+			if (key >= KEY_VOXELISE_ENABLE_DENOMINATOR*1000) {
 				bool b;
 				if(stream_cast(b,value))
 					return false;
-//				if (b && !rsdIncoming->enabledIons[key - KEY_VOXEL_ENABLE_DENOMINATOR*1000]) {
+//				if (b && !rsdIncoming->enabledIons[key - KEY_VOXELISE_ENABLE_DENOMINATOR*1000]) {
 //					return false;
 //				}
-				enabledIons[1][key - KEY_VOXEL_ENABLE_DENOMINATOR*1000]=b;
+				enabledIons[1][key - KEY_VOXELISE_ENABLE_DENOMINATOR*1000]=b;
 				if (!b) {
 					denominatorAll = false;
 				}
 				needUpdate=true;			
-			} else if (key >= KEY_VOXEL_ENABLE_NUMERATOR*1000) {
+				clearCache();
+			} else if (key >= KEY_VOXELISE_ENABLE_NUMERATOR*1000) {
 				bool b;
 				if(stream_cast(b,value))
 					return false;
-//				if (b && !rsdIncoming->enabledIons[key - KEY_VOXEL_ENABLE_NUMERATOR*1000]) {
+//				if (b && !rsdIncoming->enabledIons[key - KEY_VOXELISE_ENABLE_NUMERATOR*1000]) {
 //					return false;
 //				}
-				enabledIons[0][key - KEY_VOXEL_ENABLE_NUMERATOR*1000]=b;
+				enabledIons[0][key - KEY_VOXELISE_ENABLE_NUMERATOR*1000]=b;
 				if (!b) {
 					numeratorAll = false;
 				}
 				needUpdate=true;			
+					clearCache();
 			}
 			else
 			{
@@ -860,14 +1216,14 @@ std::string  VoxeliseFilter::getErrString(unsigned int code) const
 {
 	switch(code)
 	{
-		case VOXEL_ABORT_ERR:
-		{
-			return std::string("Voxelisation aborted");
-		}
-		case VOXEL_BOUNDS_INVALID_ERR:
-		{
-			return std::string("Voxelisation bounds are invalid");
-		}
+		case VOXELISE_ABORT_ERR:
+			return std::string(TRANS("Voxelisation aborted"));
+		case VOXELISE_MEMORY_ERR:
+			return std::string(TRANS("Out of memory"));
+		case VOXELISE_CONVOLVE_ERR:
+			return std::string(TRANS("Unable to perform filter convolution"));
+		case VOXELISE_BOUNDS_INVALID_ERR:
+			return std::string(TRANS("Voxelisation bounds are invalid"));
 	}	
 	
 	return std::string("BUG! Should not see this (VoxeliseFilter)");
@@ -886,6 +1242,20 @@ bool VoxeliseFilter::writeState(std::ofstream &f,unsigned int format, unsigned i
 			f << tabs(depth+1) << "<nbins values=\""<<nBins[0] << ","<<nBins[1]<<","<<nBins[2] << "\"/>"  << endl;
 			f << tabs(depth+1) << "<binwidth values=\""<<binWidth[0] << ","<<binWidth[1]<<","<<binWidth[2] << "\"/>"  << endl;
 			f << tabs(depth+1) << "<normalisetype value=\""<<normaliseType << "\"/>"  << endl;
+			f << tabs(depth+1) << "<enabledions>" << endl;
+
+			f << tabs(depth+2) << "<numerator>" << endl;
+			for(unsigned int ui=0;ui<enabledIons[0].size(); ui++)
+				f << tabs(depth+3) << "<enabled value=\"" << (enabledIons[0][ui]?1:0) << "\"/>" << endl;
+			f << tabs(depth+2) << "</numerator>" << endl;
+
+			f << tabs(depth+2) << "<denominator>" << endl;
+			for(unsigned int ui=0;ui<enabledIons[1].size(); ui++)
+				f << tabs(depth+3) << "<enabled value=\"" << (enabledIons[1][ui]?1:0) << "\"/>" << endl;
+			f << tabs(depth+2) << "</denominator>" << endl;
+
+			f << tabs(depth+1) << "</enabledions>" << endl;
+
 			f << tabs(depth+1) << "<representation value=\""<<representation << "\"/>" << endl;
 			f << tabs(depth+1) << "<colour r=\"" <<  r<< "\" g=\"" << g << "\" b=\"" <<b
 				<< "\" a=\"" << a << "\"/>" <<endl;
@@ -905,7 +1275,8 @@ bool VoxeliseFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFile
 	using std::string;
 	string tmpStr;
 	xmlChar *xmlString;
-	
+	stack<xmlNodePtr> nodeStack;
+
 	//Retrieve user string
 	//===
 	if(XMLHelpFwdToElem(nodePtr,"userstring"))
@@ -967,9 +1338,90 @@ bool VoxeliseFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFile
 	//Retrieve normaliseType
 	if(!XMLGetNextElemAttrib(nodePtr,normaliseType,"normalisetype","value"))
 		return false;
-	if(normaliseType >= VOXEL_NORMALISETYPE_MAX)
+	if(normaliseType >= VOXELISE_NORMALISETYPE_MAX)
 		return false;
+
+	//Look for the enabled ions bit
+	//-------	
+	//
 	
+	if(!XMLHelpFwdToElem(nodePtr,"enabledions"))
+	{
+
+		nodeStack.push(nodePtr);
+		if(!nodePtr->xmlChildrenNode)
+			return false;
+		nodePtr=nodePtr->xmlChildrenNode;
+		
+		//enabled ions for numerator
+		if(XMLHelpFwdToElem(nodePtr,"numerator"))
+			return false;
+
+		nodeStack.push(nodePtr);
+
+		if(!nodePtr->xmlChildrenNode)
+			return false;
+
+		nodePtr=nodePtr->xmlChildrenNode;
+
+		while(nodePtr)
+		{
+			char c;
+			//Retrieve representation
+			if(!XMLGetNextElemAttrib(nodePtr,c,"enabled","value"))
+				break;
+
+			if(c == '1')
+				enabledIons[0].push_back(true);
+			else
+				enabledIons[0].push_back(false);
+
+
+			nodePtr=nodePtr->next;
+		}
+
+		nodePtr=nodeStack.top();
+		nodeStack.pop();
+
+		//enabled ions for demerator
+		if(XMLHelpFwdToElem(nodePtr,"denominator"))
+			return false;
+
+
+		if(!nodePtr->xmlChildrenNode)
+			return false;
+
+		nodeStack.push(nodePtr);
+		nodePtr=nodePtr->xmlChildrenNode;
+
+		while(nodePtr)
+		{
+			char c;
+			//Retrieve representation
+			if(!XMLGetNextElemAttrib(nodePtr,c,"enabled","value"))
+				break;
+
+			if(c == '1')
+				enabledIons[1].push_back(true);
+			else
+				enabledIons[1].push_back(false);
+				
+
+			nodePtr=nodePtr->next;
+		}
+
+
+		nodeStack.pop();
+		nodePtr=nodeStack.top();
+		nodeStack.pop();
+
+		//Check that the enabled ions size makes at least some sense...
+		if(enabledIons[0].size() != enabledIons[1].size())
+			return false;
+
+	}
+
+	//-------	
 	//Retrieve representation
 	if(!XMLGetNextElemAttrib(nodePtr,representation,"representation","value"))
 		return false;
@@ -991,8 +1443,8 @@ bool VoxeliseFilter::readState(xmlNodePtr &nodePtr, const std::string &stateFile
 
 int VoxeliseFilter::getRefreshBlockMask() const
 {
-	//Absolutely nothing can go through this filter.
-	return STREAMTYPE_MASK_ALL;
+	//Ions, plots and voxels cannot pass through this filter
+	return STREAM_TYPE_IONS | STREAM_TYPE_PLOT | STREAM_TYPE_VOXEL;
 }
 
 int VoxeliseFilter::getRefreshEmitMask() const
