@@ -35,8 +35,7 @@ const unsigned int MAX_CALLBACK=500;
 
 using namespace std;
 
-#include <gsl/gsl_linalg.h>
-#include <gsl/gsl_eigen.h>
+
 #ifndef M_PI
 #define M_PI 3.14159265358979323846
 #endif
@@ -53,7 +52,8 @@ enum{
 	BOUND_CLIP=1,
 	BOUND_HOLD,
 	BOUND_DERIV_HOLD,
-	BOUND_MIRROR
+	BOUND_MIRROR,
+	BOUND_ZERO
 };
 
 
@@ -67,7 +67,7 @@ enum{
 	VOXELS_BAD_FILE_READ=1,
 	VOXELS_BAD_FILE_OPEN,
 	VOXELS_BAD_FILE_SIZE,
-	VOXELS_OUT_OF_MEMORY,
+	VOXELS_OUT_OF_MEMORY
 };
 
 //Must be power of two (buffer size when loading files, in sizeof(T)s)
@@ -79,13 +79,14 @@ const unsigned int ITEM_BUFFER_SIZE=65536;
 enum {
 	CLIP_NONE=0,
        CLIP_LOWER_SOUTH_WEST,
-       CLIP_UPPER_NORTH_EAST,
+       CLIP_UPPER_NORTH_EAST
 };
 
 
 enum {
 	VOXEL_ABORT_ERR,
-	VOXEL_BOUNDS_INVALID_ERR,
+	VOXEL_MEMORY_ERR,
+	VOXEL_BOUNDS_INVALID_ERR
 };
 
 /*
@@ -184,7 +185,6 @@ template<class T> class Voxels
 		 * by its ith index
 		 */
 		//const T &getDataRef(size_t i) const { return voxels[i];};
-
 		//!Returns underlying pointer --  UNSUPPORTED FUNCTION -- USE AT YOUR PERIL
 		/* No guarantees are made about world not exploding
 		 * Function use may cause operator harm! Do not use 
@@ -193,6 +193,10 @@ template<class T> class Voxels
 		 */
 		//const T *getDataPtr() const { return voxels;};
 
+		//Return a padded version of the data. Valid pads are BOUND_*
+		T getPaddedData(long long x, long long y, long long z, 
+						unsigned int padMode) const;
+
 
 		void setEntry(size_t n, const T &val) { voxels[n] = val;};
 		//!Retrieve a reference to the data ata  given position
@@ -212,7 +216,9 @@ template<class T> class Voxels
 		 * Data will *not* be initialised
 		 */
 		size_t resize(size_t newX, size_t newY, size_t newZ, const Point3D &newMinBound=Point3D(0.0f,0.0f,0.0f), const Point3D &newMaxBound=Point3D(1.0f,1.0f,1.0f));
-		
+	
+		size_t resize(const Voxels<T> &v);
+
 		//!Resize the data field, maintaining data as best as possible
 		/*! This will preserve data by resizing as much as possible 
 		 * about the origin. If the bounds are extended, the "fill" value is used
@@ -233,7 +239,17 @@ template<class T> class Voxels
 	
 		//!Convert the edge index (as generated by getEdgeIndex) into a cenre position	
 		// returns the axis value so you know edge vector too.
-		Point3D getEdgeCentre(unsigned int edgeIndex,size_t &axis) const;
+		// NOte that the value to pass as the edge index is (getEdgeIndex>>2)<<2 to
+		// make the ownership of the voxel correct 
+		void getEdgeEnds(size_t edgeIndex,Point3D &a, Point3D &b) const;
+
+
+
+		//TODO: there is duplicate code between this and getEdgeEnds. Refactor.
+		//!Return the values that are associated with the edge ends, as returned by getEdgeEnds
+		void getEdgeEndApproxVals(size_t edgeUniqId, float &a, float &b ) const;
+
+
 		//!Rebin the data by a given rate
 		/*! This will perform a quick and dirty rebin operation, where groups of datablocks 
 		 * are binned into a single cell. Number of blocks binned is rate^3. Field must be larger than rate
@@ -246,6 +262,9 @@ template<class T> class Voxels
 		 */
 		T getSum(const T &initialVal=T(0.0)) const;
 
+		//!count the number of cells with at least this intensity
+		size_t count(const T &minIntensity) const;
+
 		//!Fill all voxels with a given value
 		void fill(const T &val);
 		//!Get the bounding size
@@ -274,15 +293,6 @@ template<class T> class Voxels
 		 */
 		size_t writeFile(const char *cpFilename) const;
 
-		//! Detect the position of the peaks in the dataset -- UNTESTED/UNCONFIRMED. USE AT YOUR PERIL.
-		/*! Uses a simple gaussian smoothed SSD to locate peaks
-		 *  sigmaThresh is the sigma used to threshold the negative of the SSD.
-		 *  sigmaSmooth is the width of the gaussian kernel 
-		 *  kernelLen is the size of the gaussian kernel (eg 3 produces a 3 by 3 by 3 kernel)
-		 */
-		void peakDetect(std::vector<Point3D> &dataVec,float sigmaThresh, 
-			float sigmaSmooth, size_t kernelLen) const;
-
 		//!Run convolution over this data, placing the correlation data into "result"
 		/*! 
 		 * Datasets MUST have the same pitch (spacing) for the result to be defined
@@ -292,6 +302,15 @@ template<class T> class Voxels
 							 bool showProgress=true,size_t boundMode=BOUND_CLIP) const; 
 
 		
+		//!Similar to convolve, but faster -- only works with separable kernels.
+		//Destroys original data in process.
+		/*! 
+		 * Datasets MUST have the same pitch (spacing) for the result to be defined
+		 * template type must have a  T(0.0) constructor that intialises it to some "zero"
+		 */
+		size_t separableConvolve(const Voxels<T> &templateVec, Voxels<T> &result,
+							 bool showProgress=true,size_t boundMode=BOUND_CLIP); 
+		
 		//!Set this object to a normalised gaussian kernel, centered around the middle of the dataset
 		void setGaussianKernelCube(float stdDev, float bound, size_t sideLen);
 		
@@ -380,9 +399,11 @@ template<class T> class Voxels
 		 */
 		int countPoints( const std::vector<Point3D> &points, bool noWrap=true, bool doErase=false);
 	
-		//! divide countPoints by volume to get density
-		int calculateDensity();
+		//! Convert voxel intensity into voxel density
+		// this is done by dividing each voxel by its volume 
+		void calculateDensity();
 
+		float getBinVolume() const;
 		//!increment the position specified
 		inline void increment(size_t x, size_t y, size_t z){
 			//Typecast everything to at least 64 bit uints.
@@ -391,9 +412,11 @@ template<class T> class Voxels
 	
 		void setCallbackMethod(bool (*cb)(void)) {callback = cb;}
 
-		//!Element wise devision	
+		//!Element wise division	
 		void operator/=(const Voxels<T> &v);
-	
+
+		void operator/=(const T &v);
+
 };
 
 //!Convert one type of voxel into another by assignment operator
@@ -456,6 +479,8 @@ void Voxels<T>::clone(Voxels<T> &newVox) const
 	newVox.binCount[2]=binCount[2];
 
 	newVox.voxels.resize(voxels.size());
+	newVox.minBound=minBound;
+	newVox.maxBound=maxBound;
 
 	std::copy(voxels.begin(),voxels.end(),newVox.voxels.begin());
 
@@ -549,24 +574,24 @@ size_t Voxels<T>::getEdgeIndex(size_t x,size_t y, size_t z, unsigned int index)
 
 	//  		    ->ASCII ART TIME<-
 	// In each individual cube, the offsets look like this:
-	//		------------7-----------
-	//		\		       |\ .
-	//		|\ 		       | \ .
-	//		| 10		       |  11
-	//		|  \		       |   \ .
-	//		|   \		       |    \ .
-	//		|    \ --------6-------------|
-	//		|     |                |     |
-	//              2     |                3     |
-	//		|     |                |     |
-	//		|     |                |     |
-	//		|     |                |     |
-	//		|     0                |     1
-	//		\-----|----5-----------      |
-	//	 	 \    |                 \    |
-	//	 	  8   |                  9   |
-	//	 	   \  |                   \  |
-	//		    \ |---------4------------
+        //		------------7-----------
+        //		\		       |\ .
+        //		|\ 		       | \ .
+        //		| 10		       |  11
+        //		|  \		       |   \ .
+        //		|   \		       |    \ .
+        //		|    \ --------6-------------|
+        //		|     |                |     |
+        //	      2 |                3     |     |
+        //		|     |                |     |
+        //		|     |                |     |
+        //		|     |                |     |
+        //		|     0                |     1
+        //		\-----|----5-----------      |
+        //	 	 \    |                 \    |
+        //	 	  8   |                  9   |
+        //	 	   \  |                   \  |
+        //		    \ |---------4------------
 	//
 	//   	^x
 	//  z|\	|
@@ -625,61 +650,258 @@ size_t Voxels<T>::getEdgeIndex(size_t x,size_t y, size_t z, unsigned int index)
 		//--
 	}
 
-	size_t axis=index/4;
 
-	return y + z*(binCount[0]+1) + x*(binCount[0]+1)*(binCount[1]+1) +
-			axis*(binCount[0]+1)*(binCount[1]+1)*(binCount[2]+1);
+	size_t result = 12*(z + y*(binCount[2]+1) + x*(binCount[2]+1)*(binCount[1]+1)) +
+	index;
+	
+	return result;
 
 }
 
 template<class T>
-Point3D Voxels<T>::getEdgeCentre(unsigned int edgeIndex, size_t &axis) const
+void Voxels<T>::getEdgeEnds(size_t edgeUniqId, Point3D &a, Point3D &b ) const
 {
-	//Invert the mapping generated by the edgeIndex function
+	//Invert the mapping generated by the edgeUniqId function
 	//to retrieve the XYZ and axis values
 	size_t x,y,z;
+
+	int index; // Per voxel edge number. See ascii art in getEdgeidx
+	index=edgeUniqId %12;
+
+	//Drop the non-owner part of the voxel
+	index/=4;
+	index*=4;
+
+	size_t tmp;
+	tmp=edgeUniqId;
+	tmp/=12; // shift out the index multiplier
+
+	x = tmp/((binCount[2]+1)*(binCount[1]+1));
+	tmp-=x*((binCount[2]+1)*(binCount[1]+1));
+
+	y=tmp/(binCount[2]+1);
+	tmp-=y*(binCount[2]+1);
+
+	z=tmp;
+
+
+
+
+	ASSERT(x< binCount[0]+1 && y<binCount[1]+1 && z<binCount[2]+1);
+
+	Point3D delta=getPitch();
+	Point3D cellCentre=getPoint(x,y,z);
 	
-	axis=edgeIndex/((binCount[0]+1)*(binCount[1]+1)*(binCount[2]+1));
-	edgeIndex-=axis*(binCount[0]+1)*(binCount[1]+1)*(binCount[2]+1);
 
-	ASSERT(axis < 3);
-
-	//The XY and Z offsets can extend slightly  (by one cell) beyond the dataset
-	x= edgeIndex/((binCount[0]+1)*(binCount[1]+1));
-	edgeIndex-=x*(binCount[0]+1)*(binCount[1]+1);
-
-	ASSERT(x <= binCount[0]);
-
-	y=edgeIndex/(binCount[0]+1);
-	edgeIndex-=y*(binCount[0]+1);
-
-	ASSERT(y <= binCount[1]);
-	z=edgeIndex;
-	
-	ASSERT(z <= binCount[0]);
-
-	Point3D cellCentre;
-	cellCentre = Point3D((float)x/(float)binCount[0]*(maxBound[0]-minBound[0]) + minBound[0],
-			(float)y/(float)binCount[1]*(maxBound[1]-minBound[1]) + minBound[1],
-			(float)z/(float)binCount[2]*(maxBound[2]-minBound[2]) + minBound[2]);
-
-	
-	Point3D delta= maxBound-minBound;
-	delta[0]/=binCount[0];
-	delta[1]/=binCount[1];
-	delta[2]/=binCount[2];
-	switch(axis)
+	//Generate ends of edge, as seen in ascii diagram in uniqueID
+	switch(index)
 	{
 		case 0:
-			return cellCentre - Point3D(delta[0],0,0);
-		case 1:
-			return cellCentre - Point3D(0,delta[1],0);
-		case 2:
-			return cellCentre - Point3D(0,0,delta[2]);
+			//|| x, lo y, lo z.	
+			a=cellCentre;
+			b=cellCentre + Point3D(delta[0],0,0);
+			break;
+
+		case 4:
+			//|| y, low x, low z.	
+			a=cellCentre;
+			b=cellCentre + Point3D(0,delta[1],0);
+			break;
+		case 8:
+			//|| z, lo x, lo y.	
+			a=cellCentre; 
+			b=cellCentre + Point3D(0,0,delta[2]);
+			break;
+		default:
+			ASSERT(false);
 	}
 
 
-	ASSERT(false);
+#ifdef DEBUG
+	BoundCube bc;
+	bc.setBounds(getMinBounds(), getMaxBounds());
+	bc.expand(sqrt(std::numeric_limits<float>::epsilon()));
+	ASSERT(bc.containsPt(a) && bc.containsPt(b));
+#endif
+}
+
+template<class T>
+void Voxels<T>::getEdgeEndApproxVals(size_t edgeUniqId, float &a, float &b ) const
+{
+	//Invert the mapping generated by the edgeUniqId function
+	//to retrieve the XYZ and axis values
+	size_t x,y,z;
+
+	int index; // Per voxel edge number. See ascii art in getEdgeidx
+	index=edgeUniqId %12;
+
+	size_t tmp;
+	tmp=edgeUniqId;
+	tmp/=12; // shift out the index multiplier
+
+	x = tmp/((binCount[2]+1)*(binCount[1]+1));
+	tmp-=x*((binCount[2]+1)*(binCount[1]+1));
+
+	y=tmp/(binCount[2]+1);
+	tmp-=y*(binCount[2]+1);
+
+	z=tmp;
+
+
+	ASSERT(x< binCount[0]+1 && y<binCount[1]+1 && z<binCount[2]+1);
+
+
+	//undo the index shuffle from getEdgeIndex
+	switch(index)
+	{
+		//X axis aligned
+		//--
+		case 0:
+			break;	
+		case 1:
+			y--; // one across in Y
+			break;	
+		case 2:
+			z--;//One across in Z
+			break;	
+		case 3:
+			y--;
+			z--;
+			break;	
+		//--
+
+		//Y Axis aligned
+		//--
+		case 4:
+			break;	
+		case 5:
+			z--;
+			break;	
+		case 6:
+			x--;
+			break;	
+		case 7:
+			z--;
+			x--;
+			break;	
+		//--
+		
+		//Z Axis aligned
+		//--
+		case 8:
+			break;	
+		case 9:
+			y--;
+			break;	
+		case 10:
+			x--;
+			break;	
+		case 11:
+			x--;
+			y--;
+			break;	
+		//--
+	}
+	//The values used here are for the "dual" grid 
+	//of voxels centres. So there are at most 2
+	//voxels sharing an edge	
+	switch(index)
+	{
+		case 0:
+		{
+			//Up we go
+			a= getData(x,y,z);
+			b= getData(x+1,y,z);
+			break;
+		}
+		case 1:
+		{
+			//
+			a= getData(x,y+1,z);
+			b= getData(x+1,y+1,z);
+			
+			break;
+		}
+		case 2:
+		{
+			a= getData(x,y,z+1);
+			b= getData(x+1,y,z+1);
+			
+			break;
+		}
+		case 3:
+		{
+			//
+			a= getData(x,y+1,z+1);
+			b= getData(x+1,y+1,z+1);
+			
+			break;
+		}
+		case 4:
+		{
+			a= getData(x,y,z);
+			b= getData(x,y+1,z);
+			//
+			
+			break;
+		}
+		case 5:
+		{
+			a= getData(x,y,z+1);
+			b= getData(x,y+1,z+1);
+			
+			break;
+		}
+		case 6:
+		{
+			a= getData(x+1,y,z);
+			b= getData(x+1,y+1,z);
+			
+			break;
+		}
+		case 7:
+		{
+			a= getData(x+1,y,z+1);
+			b= getData(x+1,y+1,z+1);
+			
+			break;
+		}
+		case 8:
+		{
+			//
+			a= getData(x,y,z);
+			b= getData(x,y,z+1);
+			
+			break;
+		}
+		case 9:
+		{
+			//
+			a= getData(x,y+1,z);
+			b= getData(x,y+1,z+1);
+			
+			break;
+		}
+		case 10:
+		{
+			a= getData(x+1,y,z);
+			b= getData(x+1,y,z+1);
+			//
+			
+			break;
+		}
+		case 11:
+		{
+			a= getData(x+1,y+1,z);
+			b= getData(x+1,y+1,z+1);
+			
+			break;
+		}
+		default:
+			ASSERT(false);
+			
+	}	
+
 }
 
 template<class T>
@@ -708,6 +930,12 @@ size_t Voxels<T>::resize(size_t x, size_t y, size_t z, const Point3D &newMinBoun
 	return 0;
 }
 
+template<class T>
+size_t Voxels<T>::resize(const Voxels<T> &oth)
+{
+	return resize(oth.binCount[0],oth.binCount[1],oth.binCount[2],oth.minBound,oth.maxBound);
+}
+
 template<class T>
 size_t Voxels<T>::resizeKeepData(size_t &newX, size_t &newY, size_t &newZ, 
 			unsigned int direction, const Point3D &newMinBound, const Point3D &newMaxBound, const T &fill,bool doFill)
@@ -972,45 +1200,22 @@ T Voxels<T>::getSum(const T &initialValue) const
 	return returnVal;
 }
 
+
 template<class T>
-void Voxels<T>::peakDetect(std::vector<Point3D> &dataVec,float sigmaThresh, 
-			float sigmaSmooth, size_t kernelSize) const
+size_t Voxels<T>::count(const T &minIntensity) const
 {
-	ASSERT(voxels.size());
+	size_t bins;
+	bins=binCount[0]*binCount[1]*binCount[2];
 
-	//A (very) simple peak detection algorithm
-	//This uses smoothed second differentials 
-	//to compute a response function.
-	
-	//Ensure sanity of input params
-	
-	//Is the kernel bigger than the dataset. That is not good !
-	ASSERT(kernelSize < binCount[0] || kernelSize < binCount[1] || kernelSize <binCount[2]);
-	
-	//compute the second differential
-	Voxels<T> ssd;
-	secondDifference(ssd);
-
-	Voxels<T> kernel,result;
-
-	//Kernel size must be odd
-	//confirm with bitwise and
-	ASSERT(kernelSize & 1)
-
-	//Shit. How do I make this work. The kernel must be rectangular. 
-	//So our voxels must be rectangular
-	//ASSERT(voxelsAreRectangular); FIXME
-	float kernelBound=(maxBound[0]-minBound[0])*kernelSize/binCount[0];
-	kernel.setGaussianKernelCube(sigmaSmooth,kernelBound,kernelSize);
-	//Convolve to get the smoothed second difference
-	ssd.convolve(kernel,result);
-
-	//Threshold to get the position. 
-	//Account for the offset (1 voxel) induced by central diff.
-
-
-	
+	size_t sum=0;
+#pragma omp parallel for reduction(+:sum)
+	for(size_t ui=0;ui<bins; ui++)
+	{
+		if(voxels[ui]>=minIntensity)
+			sum++;
+	}
 
+	return sum;
 }
 
 template<class T>
@@ -1140,6 +1345,217 @@ size_t Voxels<T>::convolve(const Voxels<T> &kernel, Voxels<T> &result,  bool sho
 	return 0;
 }
 
+template<class T>
+size_t Voxels<T>::separableConvolve(const Voxels<T> &kernel, Voxels<T> &result,  bool showProgress, size_t boundMode) 
+{
+	ASSERT(voxels.size());
+
+	//Loop through all of the voxel elements, setting the
+	//result voxels to the convolution of the template
+	//with the data 
+
+	if(showProgress)
+	{
+		cerr << std::endl <<  "|";
+		for(size_t ui=0; ui<100; ui++)
+			cerr << "." ; 
+		cerr << "|" << std::endl << "|";
+	}
+	
+
+	size_t progress=0;
+	size_t its=0;
+	Point3D resultMinBound, resultMaxBound;
+	size_t x,y,z;
+	long long half;
+	kernel.getSize(x,y,z);
+	half=x/2;
+	//Kernel needs to be cubic
+	ASSERT(x==y && y == z && z%2);
+
+	if(boundMode!=BOUND_CLIP)
+	{
+		
+		resultMinBound=minBound;
+		resultMaxBound=maxBound;
+
+		ASSERT(result.binCount[0] == binCount[0] &&
+				result.binCount[1] == binCount[1] &&
+				result.binCount[2] == binCount[2]);
+
+		T tally;
+
+		//Separable kernels can be repeatably convolved
+		//ie, f*g can be expressed as (f*g(x) + 
+		//
+
+
+		if(x < binCount[0]/2)
+		{	
+			//Convolve in X direction only
+			for(size_t uj=0;uj<binCount[1];uj++)
+			{
+			for(size_t uk=0;uk<binCount[2];uk++)
+			{
+				for(unsigned long long ui=0;ui<x; ui++)
+				{
+					tally=T(0.0);
+					for(unsigned long long ul=-half;ul<half; ul++)
+						tally+=(getPaddedData(ui+ul,uj,uk,boundMode))*
+								kernel.getData(ul+half,half,half);
+
+					result.setData(ui,uj,uk,tally);
+				}
+				
+				for(unsigned long long ui=binCount[0]-x;ui<binCount[0]; ui++)
+				{
+					tally=T(0.0);
+					for(unsigned long long ul=-half;ul<half; ul++)
+						tally+=(getPaddedData(ui+ul,uj,uk,boundMode))*
+								kernel.getData(ul+half,half,half);
+					result.setData(ui,uj,uk,tally);
+				}
+
+				//Unpadded section
+				for(size_t ui=x;ui<binCount[0]-x; ui++)
+				{
+					tally=T(0.0);
+					for(unsigned long long ul=-half;ul<half; ul++)
+						tally+=(getPaddedData(ui+ul,uj,uk,boundMode))*
+								kernel.getData(ul+half,half,half);
+					
+					result.setData(ui,uj,uk,result.getData(ui,uj,uk)+tally);
+				}
+			}
+
+			}
+		}
+		else
+		{
+			//FIXME: IMPLEMENT ME!!
+			ASSERT(false);
+		}
+		
+		swap(result);
+
+		if(y < binCount[1]/2)
+		{	
+			//Convolve in Y direction only
+			for(size_t ui=0;ui<binCount[0];ui++)
+			{
+			for(size_t uk=0;uk<binCount[2];uk++)
+			{
+				for(unsigned long long uj=0;uj<y; uj++)
+				{
+					tally=T(0.0);
+					for(unsigned long long ul=-half;ul<half; ul++)
+						tally+=(getPaddedData(ui,uj+ul,uk,boundMode))*
+								kernel.getData(half,half+ul,half);
+					result.setData(ui,uj,uk,tally);
+				}
+				
+				for(unsigned long long uj=binCount[1]-y;uj<binCount[1]; uj++)
+				{
+					tally=T(0.0);
+					for(unsigned long long ul=-half;ul<half; ul++)
+						tally+=(getPaddedData(ui,uj+ul,uk,boundMode))*
+								kernel.getData(half,half+ul,half);
+					result.setData(ui,uj,uk,tally);
+				}
+
+				//Unpadded section
+				for(size_t uj=y;uj<binCount[1]-y; uj++)
+				{
+					tally=T(0.0);
+					for(unsigned long long ul=-half;ul<half; ul++)
+						tally+=(getPaddedData(ui,uj+ul,uk,boundMode))*
+								kernel.getData(half,half+ul,half);
+
+					result.setData(ui,uj,uk,result.getData(ui,uj,uk)+tally);
+				}
+			}
+
+			}
+		}
+		else
+		{
+			//FIXME: IMPLEMENT ME!!
+			ASSERT(false);
+		}	
+	
+		swap(result);
+		
+		if(z < binCount[2]/2)
+		{	
+			//Convolve in Z direction onlbinCount[0]
+			for(size_t ui=0;ui<binCount[0];ui++)
+			{
+			for(size_t uj=0;uj<binCount[1];uj++)
+			{
+				for(long long uk=0;uk<z; uk++)
+				{
+					tally=T(0.0);
+					for(long long ul=-half;ul<half; ul++)
+						tally+=(getPaddedData(ui,uj,uk+ul,boundMode))*
+								kernel.getData(half,half,half+ul);
+					result.setData(ui,uj,uk,tally);
+				}
+				
+				for(long long uk=binCount[2]-z;uk<binCount[2]; uk++)
+				{
+					tally=T(0.0);
+					for(long long ul=-half;ul<half; ul++)
+						tally+=(getPaddedData(ui,uj,uk+ul,boundMode))*
+								kernel.getData(half,half,half+ul);
+					result.setData(ui,uj,uk,tally);
+				}
+
+				//Unpadded section
+				for(size_t uk=z;uk<binCount[2]-z; uk++)
+				{
+					tally=T(0.0);
+					for(long long ul=-half;ul<half; ul++)
+						tally+=(getPaddedData(ui,uj,uk+ul,boundMode))*
+								kernel.getData(half,half,half+ul);
+
+					result.setData(ui,uj,uk,result.getData(ui,uj,uk)+tally);
+				}
+			}
+
+			}
+		}
+		else
+		{
+			//FIXME: IMPLEMENT ME!!
+			ASSERT(false);
+		}	
+	}
+	else
+	{
+	
+		//Check the kernel can fit within this datasest
+		if(binCount[0] <x || binCount[1]< y || binCount[2] < z)
+			return 1;
+		resultMinBound=minBound*Point3D((float)(binCount[0]-x)/(float)binCount[0],
+			(float)(binCount[1]-y)/(float)binCount[1],(float)(binCount[2]-z)/(float)binCount[2] );
+		resultMaxBound=maxBound*Point3D((float)(binCount[0]-x)/(float)binCount[0],
+			(float)(binCount[1]-y)/(float)binCount[1],(float)(binCount[2]-z)/(float)binCount[2] );
+		//FIXME: IMPLEMENT ME!
+		ASSERT(false);
+	}
+
+
+
+
+	if(showProgress)
+	{
+		cerr << "|" << std::endl;
+	}
+	
+	return 0;
+}
+
+
 template<class T>
 T Voxels<T>::getData(size_t x, size_t y, size_t z) const
 {
@@ -1155,6 +1571,50 @@ T Voxels<T>::getData(size_t x, size_t y, size_t z) const
 	return	voxels[off]; 
 }
 
+template<class T>
+T Voxels<T>::getPaddedData(long long x, long long y, long long z, unsigned int padMode) const
+{
+	if(x >=0 && x<binCount[0] && y>=0 && y<binCount[1] && z>=0 && z <binCount[2])
+	{
+		return getData(x,y,z);
+	}
+
+	//OK, so we are not inside the dataset -- we have to guess
+	
+	switch(padMode)
+	{
+		case BOUND_ZERO:
+			return T(0);
+		case BOUND_MIRROR:
+		{
+
+			if(x<0)
+				x=-x;
+			
+			if(y<0)
+				y=-y;
+			
+			if(z<0)
+				z=-z;
+		
+			if(x >=binCount[0])
+				x=binCount[0]-(x-binCount[0]+1);
+			if(y >=binCount[1])
+				y=binCount[1]-(y-binCount[1]+1);
+			if(z >=binCount[2])
+				z=binCount[2]-(z-binCount[2]+1);
+
+			
+			return  getData(x,y,z);
+		}
+		default:
+			//Should only get here if we have not implemented the bound mode
+			ASSERT(false);
+	}
+
+	ASSERT(false);
+}
+
 /*
 template<class T>
 const T &Voxels<T>::getDataRef(size_t x, size_t y, size_t z) const
@@ -1176,7 +1636,7 @@ void Voxels<T>::setGaussianKernelCube(float stdDev, float bound, size_t sideLen)
 	T obj;
 
 	//Equivariance gaussian.
-	float product = 1.0/(stdDev*stdDev*stdDev*sqrtf(2.0*M_PI));
+	float product = 1.0/(stdDev*stdDev*stdDev*sqrtf(powf(2.0*M_PI,3)));
 
 	float scale;
 	const size_t halfLen = sideLen/2;
@@ -1186,6 +1646,7 @@ void Voxels<T>::setGaussianKernelCube(float stdDev, float bound, size_t sideLen)
 	const float twoVariance=2.0*stdDev*stdDev;
 
 	scale=bound/sideLen;
+	scale*=scale;
 	//Loop through the data
 	//This could be sped up using decrementing whiles, 
 	//at the cost of losing the openMP bit
@@ -1597,96 +2058,6 @@ void Voxels<T>::makeSphericalKernel(size_t sideLen, float bound, const T &val, u
 	
 }
 
-template<class T>
-void Voxels<T>::moravecHarrisFeatures(Voxels<float> &eigA, Voxels<float> &eigB,Voxels<float> &eigC) const
-{
-	//3D implementation of the moravec plessy edge detector
-	eigA.resize(binCount[0]-2,binCount[1]-2,
-			binCount[2]-2,getMinBounds(), getMaxBounds());
-	eigB.resize(binCount[0]-2,binCount[1]-2,
-			binCount[2]-2,getMinBounds(), getMaxBounds());
-	eigC.resize(binCount[0]-2,binCount[1]-2,
-			binCount[2]-2,getMinBounds(), getMaxBounds());
-
-	float xPitch,yPitch,zPitch;
-
-	xPitch=(maxBound[0]-minBound[0])/binCount[0];
-	yPitch=(maxBound[1]-minBound[1])/binCount[1];
-	zPitch=(maxBound[2]-minBound[2])/binCount[2];
-
-	//Moravec derivative matrix
-	gsl_matrix *M ;
-	//Workspace for computing eigenvalues
-	gsl_eigen_symm_workspace *w;
-	//Vector for storing the eigenvalues
-	gsl_vector *v;
-	
-	M=gsl_matrix_alloc(3,3);
-	v=gsl_vector_alloc(3);
-	w= gsl_eigen_symm_alloc(3);
-
-
-	//NOTE: I have trouble using openmp to parallelise this
-	//as the M,v and w variables need ot have their own storage
-	//allocated. Could use arrays of M[],v[] and w[]  but I will worry about it another day
-	
-	//Calculate the directional derivatives
-	float deriv[3];
-	//Work out boundary
-	for(size_t ui=1; ui<binCount[0]-1; ui++)
-	{
-		for(size_t uj=1; uj<binCount[1]-1; uj++)
-		{
-			for(size_t uk=1; uk<binCount[2]-1; uk++)
-			{
-				
-				//dx by first order central difference
-				deriv[0]=T((getData(ui+1,uj,uk) + getData(ui-1,uj,uk))/(xPitch));
-				
-				//dy
-				deriv[1]=T((getData(ui,uj+1,uk) + getData(ui,uj-1,uk))/(yPitch));
-				
-				//dz
-				deriv[2]=T((getData(ui,uj,uk+1) + getData(ui,uj,uk-1))/(zPitch));
-
-				//NOTE: Harris' paper 
-				//(Harris, C. & Stephens, M. A combined corner and 
-				//edge detector Alvey vision conference, 1998, 15, 50 )
-				//suggests a way to not compute the eigenvalues 
-				//and still have a reasonable detector. This is not implemented here, but could
-				//be done for a speedup
-
-				//Construct the matrix
-				//Note that the matrix is symmetric,
-				//so really we only construct the lower triangluar component
-				gsl_matrix_set(M,0,0,(float)(deriv[0]*deriv[0]));
-				gsl_matrix_set(M,1,0,(float)(deriv[1]*deriv[0]));
-				gsl_matrix_set(M,1,1,(float)(deriv[1]*deriv[1]));
-				gsl_matrix_set(M,2,0,(float)(deriv[2]*deriv[0]));
-				gsl_matrix_set(M,2,1,(float)(deriv[2]*deriv[1]));
-				gsl_matrix_set(M,2,2,(float)(deriv[2]*deriv[2]));
-				
-
-				//Find the eigenvalues
-				//GSL reference manual: 
-				//The diagonal and lower triangular part of A 
-				//are destroyed during the computation, but the strict 
-				//upper triangular part is not referenced.
-				gsl_eigen_symm(M, v, w);
-				
-				eigA.setData(ui-1,uj-1,uk-1,gsl_vector_get(v,0));
-				eigB.setData(ui-1,uj-1,uk-1,gsl_vector_get(v,1));
-				eigC.setData(ui-1,uj-1,uk-1,gsl_vector_get(v,2));
-
-			}
-		}
-	}
-
-	gsl_eigen_symm_free(w);
-	gsl_vector_free(v);
-	gsl_matrix_free(M);
-}
-
 template<class T>
 T Voxels<T>::min() const
 {
@@ -1931,7 +2302,7 @@ int Voxels<T>::countPoints( const std::vector<IonHit> &points, bool noWrap, bool
 */
 
 template<class T>
-int Voxels<T>::calculateDensity()
+void Voxels<T>::calculateDensity()
 {
 	Point3D size = maxBound - minBound;
 	// calculate the volume of a voxel
@@ -1944,7 +2315,17 @@ int Voxels<T>::calculateDensity()
 	for(size_t ui=0; ui<voxels.size(); ui++) 
 		voxels[ui] /= volume;
 
-	return 0;
+}
+
+template<class T>
+float Voxels<T>::getBinVolume() const
+{
+	Point3D size = maxBound - minBound;
+	double volume = 1.0;
+	for (int i = 0; i < 3; i++)
+		volume *= size[i] / binCount[i];
+
+	return volume;
 }
 
 template<class T>
@@ -2153,6 +2534,7 @@ template<class T>
 void Voxels<T>::operator/=(const Voxels<T> &v)
 {
 	ASSERT(v.voxels.size() == voxels.size());
+#pragma omp parallel for
 	for (size_t i = 0; i < voxels.size(); i++)
 	{
 		if(v.voxels[i])
@@ -2161,6 +2543,16 @@ void Voxels<T>::operator/=(const Voxels<T> &v)
 			voxels[i] =0;
 	}
 }	
+
+	
+template<class T>
+void Voxels<T>::operator/=(const T &v)
+{
+	ASSERT(v);
+#pragma omp parallel for
+	for (size_t i = 0; i < voxels.size(); i++)
+		voxels[i]/=v;
+}	
 //===
 
 
